@@ -1,5 +1,5 @@
 /*
- * linux/drivers/ide/pci/aec62xx.c		Version 0.11	March 27, 2002
+ * linux/drivers/ide/aec62xx.c		Version 0.11	March 27, 2002
  *
  * Copyright (C) 1999-2002	Andre Hedrick <andre@linux-ide.org>
  *
@@ -38,12 +38,13 @@ static int aec62xx_get_info (char *buffer, char **addr, off_t offset, int count)
 	char *chipset_nums[] = {"error", "error", "error", "error",
 				"error", "error", "850UF",   "860",
 				 "860R",   "865",  "865R", "error"  };
-	int len;
+//	char *modes_33[] = {};
+//	char *modes_34[] = {};
 	int i;
 
 	for (i = 0; i < n_aec_devs; i++) {
 		struct pci_dev *dev	= aec_devs[i];
-		unsigned long iobase = pci_resource_start(dev, 4);
+		u32 iobase		= pci_resource_start(dev, 4);
 		u8 c0 = 0, c1 = 0, art	= 0;
 #ifdef DEBUG_AEC_REGS
 		u8 uart			= 0;
@@ -171,11 +172,7 @@ static int aec62xx_get_info (char *buffer, char **addr, off_t offset, int count)
 #endif /* DEBUG_AEC_REGS */
 		}
 	}
-	/* p - buffer must be less than 4k! */
-	len = (p - buffer) - offset;
-	*addr = buffer + offset;
-	
-	return len > count ? count : len;
+	return p-buffer;/* => must be less than 4k! */
 }
 #endif	/* defined(DISPLAY_AEC62xx_TIMINGS) && defined(CONFIG_PROC_FS) */
 
@@ -297,6 +294,7 @@ static int aec62xx_tune_chipset (ide_drive_t *drive, u8 speed)
 	}
 }
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 static int config_chipset_for_dma (ide_drive_t *drive)
 {
 	u8 speed = ide_dma_speed(drive, aec62xx_ratemask(drive));	
@@ -307,6 +305,7 @@ static int config_chipset_for_dma (ide_drive_t *drive)
 	(void) aec62xx_tune_chipset(drive, speed);
 	return ide_dma_enable(drive);
 }
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 
 static void aec62xx_tune_drive (ide_drive_t *drive, u8 pio)
 {
@@ -324,12 +323,13 @@ static void aec62xx_tune_drive (ide_drive_t *drive, u8 pio)
 	(void) aec62xx_tune_chipset(drive, speed);
 }
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 static int aec62xx_config_drive_xfer_rate (ide_drive_t *drive)
 {
 	ide_hwif_t *hwif	= HWIF(drive);
 	struct hd_driveid *id	= drive->id;
 
-	if ((id->capability & 1) && drive->autodma) {
+	if (id && (id->capability & 1) && drive->autodma) {
 		/* Consult the list of known "bad" drives */
 		if (hwif->ide_dma_bad_drive(drive))
 			goto fast_ata_pio;
@@ -407,6 +407,7 @@ static int aec62xx_irq_timeout (ide_drive_t *drive)
 #endif
 	return 0;
 }
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 
 static unsigned int __init init_chipset_aec62xx (struct pci_dev *dev, const char *name)
 {
@@ -414,7 +415,7 @@ static unsigned int __init init_chipset_aec62xx (struct pci_dev *dev, const char
 
 	if (dev->resource[PCI_ROM_RESOURCE].start) {
 		pci_write_config_dword(dev, PCI_ROM_ADDRESS, dev->resource[PCI_ROM_RESOURCE].start | PCI_ROM_ADDRESS_ENABLE);
-		printk(KERN_INFO "%s: ROM enabled at 0x%08lx\n", name, dev->resource[PCI_ROM_RESOURCE].start);
+		printk("%s: ROM enabled at 0x%08lx\n", name, dev->resource[PCI_ROM_RESOURCE].start);
 	}
 
 #if defined(DISPLAY_AEC62XX_TIMINGS) && defined(CONFIG_PROC_FS)
@@ -458,6 +459,7 @@ static void __init init_hwif_aec62xx (ide_hwif_t *hwif)
 	hwif->mwdma_mask = 0x07;
 	hwif->swdma_mask = 0x07;
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 	hwif->ide_dma_check	= &aec62xx_config_drive_xfer_rate;
 	hwif->ide_dma_lostirq	= &aec62xx_irq_timeout;
 	hwif->ide_dma_timeout	= &aec62xx_irq_timeout;
@@ -465,6 +467,7 @@ static void __init init_hwif_aec62xx (ide_hwif_t *hwif)
 		hwif->autodma = 1;
 	hwif->drives[0].autodma = hwif->autodma;
 	hwif->drives[1].autodma = hwif->autodma;
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 }
 
 static void __init init_dma_aec62xx (ide_hwif_t *hwif, unsigned long dmabase)
@@ -498,7 +501,7 @@ static void __init init_setup_aec62xx (struct pci_dev *dev, ide_pci_device_t *d)
 
 static void __init init_setup_aec6x80 (struct pci_dev *dev, ide_pci_device_t *d)
 {
-	unsigned long bar4reg = pci_resource_start(dev, 4);
+	u32 bar4reg = pci_resource_start(dev, 4);
 
 	if (inb(bar4reg+2) & 0x10) {
 		strcpy(d->name, "AEC6880");
@@ -513,56 +516,22 @@ static void __init init_setup_aec6x80 (struct pci_dev *dev, ide_pci_device_t *d)
 	ide_setup_pci_device(dev, d);
 }
 
-/**
- *	aec62xx_init_one	-	called when a AEC is found
- *	@dev: the aec62xx device
- *	@id: the matching pci id
- *
- *	Called when the PCI registration layer (or the IDE initialization)
- *	finds a device matching our IDE device tables.
- */
- 
-static int __devinit aec62xx_init_one(struct pci_dev *dev, const struct pci_device_id *id)
+int __init aec62xx_scan_pcidev (struct pci_dev *dev)
 {
-	ide_pci_device_t *d = &aec62xx_chipsets[id->driver_data];
+	ide_pci_device_t *d;
 
-	if (dev->device != d->device)
-		BUG();
-	d->init_setup(dev, d);
-	MOD_INC_USE_COUNT;
+	if (dev->vendor != PCI_VENDOR_ID_ARTOP)
+		return 0;
+
+	for (d = aec62xx_chipsets;
+		d && d->vendor && d->device; ++d) {
+		if (((d->vendor == dev->vendor) &&
+		     (d->device == dev->device)) &&
+		    (d->init_setup)) {
+			d->init_setup(dev, d);
+			return 1;
+		}
+	}
 	return 0;
 }
 
-static struct pci_device_id aec62xx_pci_tbl[] __devinitdata = {
-	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP850UF, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP860,   PCI_ANY_ID, PCI_ANY_ID, 0, 0, 1 },
-	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP860R,  PCI_ANY_ID, PCI_ANY_ID, 0, 0, 2 },
-	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP865,   PCI_ANY_ID, PCI_ANY_ID, 0, 0, 3 },
-	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP865R,  PCI_ANY_ID, PCI_ANY_ID, 0, 0, 4 },
-	{ 0, },
-};
-
-static struct pci_driver driver = {
-	.name		= "AEC62xx IDE",
-	.id_table	= aec62xx_pci_tbl,
-	.probe		= aec62xx_init_one,
-};
-
-static int aec62xx_ide_init(void)
-{
-	return ide_pci_register_driver(&driver);
-}
-
-static void aec62xx_ide_exit(void)
-{
-	ide_pci_unregister_driver(&driver);
-}
-
-module_init(aec62xx_ide_init);
-module_exit(aec62xx_ide_exit);
-
-MODULE_AUTHOR("Andre Hedrick");
-MODULE_DESCRIPTION("PCI driver module for ARTOP AEC62xx IDE");
-MODULE_LICENSE("GPL");
-
-EXPORT_NO_SYMBOLS;
