@@ -1,5 +1,5 @@
 /*
- * linux/drivers/ide/pci/hpt34x.c		Version 0.40	Sept 10, 2002
+ * linux/drivers/ide/hpt34x.c		Version 0.31	June. 9, 2000
  *
  * Copyright (C) 1998-2000	Andre Hedrick <andre@linux-ide.org>
  * May be copied or modified under the terms of the GNU General Public License
@@ -25,7 +25,6 @@
  */
 
 #include <linux/config.h>
-#include <linux/module.h>
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/delay.h>
@@ -58,13 +57,13 @@ static int n_hpt34x_devs;
 static int hpt34x_get_info (char *buffer, char **addr, off_t offset, int count)
 {
 	char *p = buffer;
-	int i, len;
+	int i;
 
 	p += sprintf(p, "\n                             "
 			"HPT34X Chipset.\n");
 	for (i = 0; i < n_hpt34x_devs; i++) {
 		struct pci_dev *dev = hpt34x_devs[i];
-		unsigned long bibma = pci_resource_start(dev, 4);
+		u32 bibma = pci_resource_start(dev, 4);
 		u8  c0 = 0, c1 = 0;
 
 		/*
@@ -96,11 +95,7 @@ static int hpt34x_get_info (char *buffer, char **addr, off_t offset, int count)
 	}
 	p += sprintf(p, "\n");
 
-	/* p - buffer must be less than 4k! */
-	len = (p - buffer) - offset;
-	*addr = buffer + offset;
-	
-	return len > count ? count : len;
+	return p-buffer;	/* => must be less than 4k! */
 }
 #endif  /* defined(DISPLAY_HPT34X_TIMINGS) && defined(CONFIG_PROC_FS) */
 
@@ -163,13 +158,13 @@ static void hpt34x_tune_drive (ide_drive_t *drive, u8 pio)
 	(void) hpt34x_tune_chipset(drive, (XFER_PIO_0 + pio));
 }
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 /*
  * This allows the configuration of ide_pci chipset registers
  * for cards that learn about the drive's UDMA, DMA, PIO capabilities
- * after the drive is reported by the OS.  Initially for designed for
+ * after the drive is reported by the OS.  Initally for designed for
  * HPT343 UDMA chipset by HighPoint|Triones Technologies, Inc.
  */
-
 static int config_chipset_for_dma (ide_drive_t *drive)
 {
 	u8 speed = ide_dma_speed(drive, hpt34x_ratemask(drive));
@@ -189,7 +184,7 @@ static int hpt34x_config_drive_xfer_rate (ide_drive_t *drive)
 
 	drive->init_speed = 0;
 
-	if ((id->capability & 1) && drive->autodma) {
+	if (id && (id->capability & 1) && drive->autodma) {
 		/* Consult the list of known "bad" drives */
 		if (hwif->ide_dma_bad_drive(drive))
 			goto fast_ata_pio;
@@ -216,20 +211,19 @@ try_dma_modes:
 		} else {
 			goto fast_ata_pio;
 		}
-#ifndef CONFIG_HPT34X_AUTODMA
-		return hwif->ide_dma_off_quietly(drive);
-#else
-		return hwif->ide_dma_on(drive);
-#endif
 	} else if ((id->capability & 8) || (id->field_valid & 2)) {
 fast_ata_pio:
 no_dma_set:
 		hpt34x_tune_drive(drive, 255);
 		return hwif->ide_dma_off_quietly(drive);
 	}
-	/* IORDY not supported */
-	return 0;
+
+#ifndef CONFIG_HPT34X_AUTODMA
+	return hwif->ide_dma_off_quietly(drive);
+#endif /* CONFIG_HPT34X_AUTODMA */
+	return hwif->ide_dma_on(drive);
 }
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 
 /*
  * If the BIOS does not set the IO base addaress to XX00, 343 will fail.
@@ -311,11 +305,13 @@ static void __init init_hwif_hpt34x (ide_hwif_t *hwif)
 	hwif->mwdma_mask = 0x07;
 	hwif->swdma_mask = 0x07;
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 	hwif->ide_dma_check = &hpt34x_config_drive_xfer_rate;
 	if (!noautodma)
 		hwif->autodma = (pcicmd & PCI_COMMAND_MEMORY) ? 1 : 0;
 	hwif->drives[0].autodma = hwif->autodma;
 	hwif->drives[1].autodma = hwif->autodma;
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 }
 
 static void __init init_dma_hpt34x (ide_hwif_t *hwif, unsigned long dmabase)
@@ -325,48 +321,34 @@ static void __init init_dma_hpt34x (ide_hwif_t *hwif, unsigned long dmabase)
 
 extern void ide_setup_pci_device(struct pci_dev *, ide_pci_device_t *);
 
-static int __devinit hpt34x_init_one(struct pci_dev *dev, const struct pci_device_id *id)
+static void __init init_setup_hpt34x (struct pci_dev *dev, ide_pci_device_t *d)
 {
-	ide_pci_device_t *d = &hpt34x_chipsets[id->driver_data];
-	static char *chipset_names[] = {"HPT343", "HPT345"};
+	char *chipset_names[] = {"HPT343", "HPT345"};
 	u16 pcicmd = 0;
 
 	pci_read_config_word(dev, PCI_COMMAND, &pcicmd);
 
-	d->name = chipset_names[(pcicmd & PCI_COMMAND_MEMORY) ? 1 : 0];
+	strcpy(d->name, chipset_names[(pcicmd & PCI_COMMAND_MEMORY) ? 1 : 0]);
 	d->bootable = (pcicmd & PCI_COMMAND_MEMORY) ? OFF_BOARD : NEVER_BOARD;
 
 	ide_setup_pci_device(dev, d);
-	MOD_INC_USE_COUNT;
+}
+
+int  __init hpt34x_scan_pcidev (struct pci_dev *dev)
+{
+	ide_pci_device_t *d;
+
+	if (dev->vendor != PCI_VENDOR_ID_TTI)
+		return 0;
+
+	for (d = hpt34x_chipsets; d && d->vendor && d->device; ++d) {
+		if (((d->vendor == dev->vendor) &&
+		     (d->device == dev->device)) &&
+		    (d->init_setup)) {
+			d->init_setup(dev, d);
+			return 1;
+		}
+	}
 	return 0;
 }
 
-static struct pci_device_id hpt34x_pci_tbl[] __devinitdata = {
-	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT343, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
-	{ 0, },
-};
-
-static struct pci_driver driver = {
-	.name		= "HPT34x IDE",
-	.id_table	= hpt34x_pci_tbl,
-	.probe		= hpt34x_init_one,
-};
-
-static int hpt34x_ide_init(void)
-{
-	return ide_pci_register_driver(&driver);
-}
-
-static void hpt34x_ide_exit(void)
-{
-	ide_pci_unregister_driver(&driver);
-}
-
-module_init(hpt34x_ide_init);
-module_exit(hpt34x_ide_exit);
-
-MODULE_AUTHOR("Andre Hedrick");
-MODULE_DESCRIPTION("PCI driver module for Highpoint 34x IDE");
-MODULE_LICENSE("GPL");
-
-EXPORT_NO_SYMBOLS;
