@@ -29,7 +29,6 @@
  */
 
 #include <linux/config.h>
-#include <linux/module.h>
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/ioport.h>
@@ -151,6 +150,7 @@ static int it8172_tune_chipset (ide_drive_t *drive, u8 xferspeed)
      */
     
 	switch(speed) {
+#ifdef CONFIG_BLK_DEV_IDEDMA
 		case XFER_UDMA_4:
 		case XFER_UDMA_2:	//u_speed = 2 << (drive->dn * 4); break;
 		case XFER_UDMA_5:
@@ -161,6 +161,7 @@ static int it8172_tune_chipset (ide_drive_t *drive, u8 xferspeed)
 		case XFER_MW_DMA_1:
 		case XFER_MW_DMA_0:
 		case XFER_SW_DMA_2:	break;
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 		case XFER_PIO_4:
 		case XFER_PIO_3:
 		case XFER_PIO_2:
@@ -181,6 +182,7 @@ static int it8172_tune_chipset (ide_drive_t *drive, u8 xferspeed)
 	return (ide_config_drive_speed(drive, speed));
 }
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 static int it8172_config_chipset_for_dma (ide_drive_t *drive)
 {
 	u8 speed = ide_dma_speed(drive, it8172_ratemask(drive));
@@ -201,7 +203,7 @@ static int it8172_config_drive_xfer_rate (ide_drive_t *drive)
 
 	drive->init_speed = 0;
 
-	if ((id->capability & 1) && drive->autodma) {
+	if (id && (id->capability & 1) && drive->autodma) {
 		/* Consult the list of known "bad" drives */
 		if (hwif->ide_dma_bad_drive(drive))
 			goto fast_ata_pio;
@@ -228,16 +230,15 @@ try_dma_modes:
 		} else {
 			goto fast_ata_pio;
 		}
-		return hwif->ide_dma_on(drive);
 	} else if ((id->capability & 8) || (id->field_valid & 2)) {
 fast_ata_pio:
 no_dma_set:
 		it8172_tune_drive(drive, 5);
 		return hwif->ide_dma_off_quietly(drive);
 	}
-	/* IORDY not supported */
-	return 0;
+	return hwif->ide_dma_on(drive);
 }
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 
 static unsigned int __init init_chipset_it8172 (struct pci_dev *dev, const char *name)
 {
@@ -280,11 +281,13 @@ static void __init init_hwif_it8172 (ide_hwif_t *hwif)
 	hwif->mwdma_mask = 0x06;
 	hwif->swdma_mask = 0x04;
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 	hwif->ide_dma_check = &it8172_config_drive_xfer_rate;
 	if (!noautodma)
 		hwif->autodma = 1;
 	hwif->drives[0].autodma = hwif->autodma;
 	hwif->drives[1].autodma = hwif->autodma;
+#endif /* !CONFIG_BLK_DEV_IDEDMA */
 }
 
 static void __init init_dma_it8172 (ide_hwif_t *hwif, unsigned long dmabase)
@@ -294,43 +297,29 @@ static void __init init_dma_it8172 (ide_hwif_t *hwif, unsigned long dmabase)
 
 extern void ide_setup_pci_device(struct pci_dev *, ide_pci_device_t *);
 
-static int __devinit it8172_init_one(struct pci_dev *dev, const struct pci_device_id *id)
+static void __init init_setup_it8172 (struct pci_dev *dev, ide_pci_device_t *d)
 {
-	ide_pci_device_t *d = &it8172_chipsets[id->driver_data];
         if ((!(PCI_FUNC(dev->devfn) & 1) ||
             (!((dev->class >> 8) == PCI_CLASS_STORAGE_IDE))))
-                return 1; /* IT8172 is more than only a IDE controller */
+                return; /* IT8172 is more than only a IDE controller */
 	ide_setup_pci_device(dev, d);
-	MOD_INC_USE_COUNT;
+}
+
+static int __init it8172_scan_pcidev (struct pci_dev *dev)
+{
+	ide_pci_device_t *d;
+
+	if (dev->vendor != PCI_VENDOR_ID_ITE)
+		return 0;
+
+	for (d = it8172_chipsets; d && d->vendor && d->device; ++d) {
+		if (((d->vendor == dev->vendor) &&
+		     (d->device == dev->device)) &&
+		    (d->init_setup)) {
+			d->init_setup(dev, d);
+			return 1;
+		}
+	}
 	return 0;
 }
 
-static struct pci_device_id it8172_pci_tbl[] __devinitdata = {
-	{ PCI_VENDOR_ID_ITE, PCI_DEVICE_ID_ITE_IT8172G, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
-	{ 0, },
-};
-
-static struct pci_driver driver = {
-	.name		= "IT8172IDE",
-	.id_table	= it8172_pci_tbl,
-	.probe		= it8172_init_one,
-};
-
-static int it8172_ide_init(void)
-{
-	return ide_pci_register_driver(&driver);
-}
-
-static void it8172_ide_exit(void)
-{
-	ide_pci_unregister_driver(&driver);
-}
-
-module_init(it8172_ide_init);
-module_exit(it8172_ide_exit);
-
-MODULE_AUTHOR("SteveL@mvista.com");
-MODULE_DESCRIPTION("PCI driver module for ITE 8172 IDE");
-MODULE_LICENSE("GPL");
-
-EXPORT_NO_SYMBOLS;
