@@ -1,5 +1,5 @@
 /*
- * linux/drivers/ide/pci/ns87415.c		Version 2.00  Sep. 10, 2002
+ * linux/drivers/ide/ns87415.c		Version 1.01  Mar. 18, 2000
  *
  * Copyright (C) 1997-1998	Mark Lord <mlord@pobox.com>
  * Copyright (C) 1998		Eddie C. Dost <ecd@skynet.be>
@@ -9,7 +9,6 @@
  */
 
 #include <linux/config.h>
-#include <linux/module.h>
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/timer.h>
@@ -83,6 +82,7 @@ static void ns87415_selectproc (ide_drive_t *drive)
 	ns87415_prepare_drive (drive, drive->using_dma);
 }
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 static int ns87415_ide_dma_end (ide_drive_t *drive)
 {
 	ide_hwif_t      *hwif = HWIF(drive);
@@ -131,6 +131,7 @@ static int ns87415_ide_dma_check (ide_drive_t *drive)
 		return HWIF(drive)->ide_dma_off_quietly(drive);
 	return __ide_dma_check(drive);
 }
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 
 static void __init init_hwif_ns87415 (ide_hwif_t *hwif)
 {
@@ -207,6 +208,7 @@ static void __init init_hwif_ns87415 (ide_hwif_t *hwif)
 	if (!hwif->dma_base)
 		return;
 
+#ifdef CONFIG_BLK_DEV_IDEDMA
 	hwif->OUTB(0x60, hwif->dma_status);
 	hwif->ide_dma_read = &ns87415_ide_dma_read;
 	hwif->ide_dma_write = &ns87415_ide_dma_write;
@@ -217,6 +219,7 @@ static void __init init_hwif_ns87415 (ide_hwif_t *hwif)
 		hwif->autodma = 1;
 	hwif->drives[0].autodma = hwif->autodma;
 	hwif->drives[1].autodma = hwif->autodma;
+#endif /* CONFIG_BLK_DEV_IDEDMA */
 }
 
 static void __init init_dma_ns87415 (ide_hwif_t *hwif, unsigned long dmabase)
@@ -226,42 +229,26 @@ static void __init init_dma_ns87415 (ide_hwif_t *hwif, unsigned long dmabase)
 
 extern void ide_setup_pci_device(struct pci_dev *, ide_pci_device_t *);
 
-static int __devinit ns87415_init_one(struct pci_dev *dev, const struct pci_device_id *id)
+static void __init init_setup_ns87415 (struct pci_dev *dev, ide_pci_device_t *d)
 {
-	ide_pci_device_t *d = &ns87415_chipsets[id->driver_data];
-	if (dev->device != d->device)
-		BUG();
 	ide_setup_pci_device(dev, d);
-	MOD_INC_USE_COUNT;
+}
+
+int __init ns87415_scan_pcidev (struct pci_dev *dev)
+{
+	ide_pci_device_t *d;
+
+	if (dev->vendor != PCI_VENDOR_ID_NS)
+		return 0;
+
+	for (d = ns87415_chipsets; d && d->vendor && d->device; ++d) {
+		if (((d->vendor == dev->vendor) &&
+		     (d->device == dev->device)) &&
+		    (d->init_setup)) {
+			d->init_setup(dev, d);
+			return 1;
+		}
+	}
 	return 0;
 }
 
-static struct pci_device_id ns87415_pci_tbl[] __devinitdata = {
-	{ PCI_VENDOR_ID_NS, PCI_DEVICE_ID_NS_87415, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
-	{ 0, },
-};
-
-static struct pci_driver driver = {
-	.name		= "NS87415IDE",
-	.id_table	= ns87415_pci_tbl,
-	.probe		= ns87415_init_one,
-};
-
-static int ns87415_ide_init(void)
-{
-	return ide_pci_register_driver(&driver);
-}
-
-static void ns87415_ide_exit(void)
-{
-	ide_pci_unregister_driver(&driver);
-}
-
-module_init(ns87415_ide_init);
-module_exit(ns87415_ide_exit);
-
-MODULE_AUTHOR("Mark Lord, Eddie Dost, Andre Hedrick");
-MODULE_DESCRIPTION("PCI driver module for NS87415 IDE");
-MODULE_LICENSE("GPL");
-
-EXPORT_NO_SYMBOLS;
