@@ -205,9 +205,6 @@
 extern void (*sa1100fb_backlight_power)(int on);
 extern void (*sa1100fb_lcd_power)(int on);
 
-void (*sa1100fb_blank_helper)(int blank);
-EXPORT_SYMBOL(sa1100fb_blank_helper);
-
 /*
  * IMHO this looks wrong.  In 8BPP, length should be 8.
  */
@@ -262,7 +259,7 @@ static struct sa1100fb_mach_info pal_info __initdata = {
 #endif
 #endif
 
-#ifdef CONFIG_SA1100_H3XXX
+#ifdef CONFIG_SA1100_H3800
 static struct sa1100fb_mach_info h3800_info __initdata = {
 	pixclock:	174757, 	bpp:		16,
 	xres:		320,		yres:		240,
@@ -274,9 +271,12 @@ static struct sa1100fb_mach_info h3800_info __initdata = {
 	sync:		0,		cmap_static:	1,
 
 	lccr0:		LCCR0_Color | LCCR0_Sngl | LCCR0_Act,
-	lccr3:		LCCR3_ACBsCntOff | LCCR3_PixFlEdg | LCCR3_OutEnH,
+	lccr3:		LCCR3_ACBsDiv(2) | LCCR3_PixRsEdg | LCCR3_OutEnH |
+			LCCR3_ACBsCntOff,
 };
+#endif
 
+#ifdef CONFIG_SA1100_H3600
 static struct sa1100fb_mach_info h3600_info __initdata = {
 	pixclock:	174757, 	bpp:		16,
 	xres:		320,		yres:		240,
@@ -288,7 +288,8 @@ static struct sa1100fb_mach_info h3600_info __initdata = {
 	sync:		0,		cmap_static:	1,
 
 	lccr0:		LCCR0_Color | LCCR0_Sngl | LCCR0_Act,
-	lccr3:		LCCR3_ACBsCntOff | LCCR3_OutEnH | LCCR3_PixFlEdg,
+	lccr3:		LCCR3_ACBsDiv(2) | LCCR3_PixRsEdg | LCCR3_OutEnH |
+			LCCR3_ACBsCntOff,
 };
 
 static struct sa1100fb_rgb h3600_rgb_16 = {
@@ -297,7 +298,9 @@ static struct sa1100fb_rgb h3600_rgb_16 = {
 	blue:	{ offset: 1,  length: 4, },
 	transp: { offset: 0,  length: 0, },
 };
+#endif
 
+#ifdef CONFIG_SA1100_H3100
 static struct sa1100fb_mach_info h3100_info __initdata = {
 	pixclock:	406977, 	bpp:		4,
 	xres:		320,		yres:		240,
@@ -680,14 +683,18 @@ sa1100fb_get_machine_info(struct sa1100fb_info *fbi)
 #endif
 	}
 #endif
-#ifdef CONFIG_SA1100_H3XXX
+#ifdef CONFIG_SA1100_H3100
+	if (machine_is_h3100()) {
+		inf = &h3100_info;
+	}
+#endif
+#ifdef CONFIG_SA1100_H3600
 	if (machine_is_h3600()) {
 		inf = &h3600_info;
 		fbi->rgb[RGB_16] = &h3600_rgb_16;
 	}
-	if (machine_is_h3100()) {
-		inf = &h3100_info;
-	}
+#endif
+#ifdef CONFIG_SA1100_H3800
 	if (machine_is_h3800()) {
 		inf = &h3800_info;
 	}
@@ -1258,6 +1265,69 @@ sa1100fb_get_cmap(struct fb_cmap *cmap, int kspc, int con, struct fb_info *info)
 	return 0;
 }
 
+/*
+ * Formal definition of the VESA spec:
+ *  On
+ *  	This refers to the state of the display when it is in full operation
+ *  Stand-By
+ *  	This defines an optional operating state of minimal power reduction with
+ *  	the shortest recovery time
+ *  Suspend
+ *  	This refers to a level of power management in which substantial power
+ *  	reduction is achieved by the display.  The display can have a longer 
+ *  	recovery time from this state than from the Stand-by state
+ *  Off
+ *  	This indicates that the display is consuming the lowest level of power
+ *  	and is non-operational. Recovery from this state may optionally require
+ *  	the user to manually power on the monitor
+ *
+ *  Now, the fbdev driver adds an additional state, (blank), where they
+ *  turn off the video (maybe by colormap tricks), but don't mess with the
+ *  video itself: think of it semantically between on and Stand-By.
+ *
+ *  So here's what we should do in our fbdev blank routine:
+ *
+ *  	VESA_NO_BLANKING (mode 0)	Video on,  front/back light on
+ *  	VESA_VSYNC_SUSPEND (mode 1)  	Video on,  front/back light off
+ *  	VESA_HSYNC_SUSPEND (mode 2)  	Video on,  front/back light off
+ *  	VESA_POWERDOWN (mode 3)		Video off, front/back light off
+ *
+ *  This will match the matrox implementation.
+ */
+/*
+ * sa1100fb_blank():
+ *	Blank the display by setting all palette values to zero.  Note, the 
+ * 	12 and 16 bpp modes don't really use the palette, so this will not
+ *      blank the display in all modes.  
+ */
+static int sa1100fb_blank(int blank, struct fb_info *info)
+{
+	struct sa1100fb_info *fbi = (struct sa1100fb_info *)info;
+	int i;
+
+	DPRINTK("sa1100fb_blank: blank=%d info->modename=%s\n", blank,
+		fbi->fb.modename);
+
+	switch (blank) {
+	case VESA_POWERDOWN:
+	case VESA_VSYNC_SUSPEND:
+	case VESA_HSYNC_SUSPEND:
+		if (fbi->fb.disp->visual == FB_VISUAL_PSEUDOCOLOR ||
+		    fbi->fb.disp->visual == FB_VISUAL_STATIC_PSEUDOCOLOR)
+			for (i = 0; i < fbi->palette_size; i++)
+				sa1100fb_setpalettereg(i, 0, 0, 0, 0, info);
+		sa1100fb_schedule_task(fbi, C_DISABLE);
+		break;
+
+	case VESA_NO_BLANKING:
+		if (fbi->fb.disp->visual == FB_VISUAL_PSEUDOCOLOR ||
+		    fbi->fb.disp->visual == FB_VISUAL_STATIC_PSEUDOCOLOR)
+			fb_set_cmap(&fbi->fb.cmap, 1, info);
+		sa1100fb_schedule_task(fbi, C_ENABLE);
+	}
+	return 0;
+}
+
 static struct fb_ops sa1100fb_ops = {
 	owner:		THIS_MODULE,
 	fb_get_fix:	sa1100fb_get_fix,
@@ -1321,73 +1391,6 @@ static int sa1100fb_switch(int con, struct fb_info *info)
 	return 0;
 }
 
-/*
- * Formal definition of the VESA spec:
- *  On
- *  	This refers to the state of the display when it is in full operation
- *  Stand-By
- *  	This defines an optional operating state of minimal power reduction with
- *  	the shortest recovery time
- *  Suspend
- *  	This refers to a level of power management in which substantial power
- *  	reduction is achieved by the display.  The display can have a longer 
- *  	recovery time from this state than from the Stand-by state
- *  Off
- *  	This indicates that the display is consuming the lowest level of power
- *  	and is non-operational. Recovery from this state may optionally require
- *  	the user to manually power on the monitor
- *
- *  Now, the fbdev driver adds an additional state, (blank), where they
- *  turn off the video (maybe by colormap tricks), but don't mess with the
- *  video itself: think of it semantically between on and Stand-By.
- *
- *  So here's what we should do in our fbdev blank routine:
- *
- *  	VESA_NO_BLANKING (mode 0)	Video on,  front/back light on
- *  	VESA_VSYNC_SUSPEND (mode 1)  	Video on,  front/back light off
- *  	VESA_HSYNC_SUSPEND (mode 2)  	Video on,  front/back light off
- *  	VESA_POWERDOWN (mode 3)		Video off, front/back light off
- *
- *  This will match the matrox implementation.
- */
-/*
- * sa1100fb_blank():
- *	Blank the display by setting all palette values to zero.  Note, the 
- * 	12 and 16 bpp modes don't really use the palette, so this will not
- *      blank the display in all modes.  
- */
-static int sa1100fb_blank(int blank, struct fb_info *info)
-{
-	struct sa1100fb_info *fbi = (struct sa1100fb_info *)info;
-	int i;
-
-	DPRINTK("sa1100fb_blank: blank=%d info->modename=%s\n", blank,
-		fbi->fb.modename);
-
-	switch (blank) {
-	case VESA_POWERDOWN:
-	case VESA_VSYNC_SUSPEND:
-	case VESA_HSYNC_SUSPEND:
-		if (fbi->fb.disp->visual == FB_VISUAL_PSEUDOCOLOR ||
-		    fbi->fb.disp->visual == FB_VISUAL_STATIC_PSEUDOCOLOR)
-			for (i = 0; i < fbi->palette_size; i++)
-				sa1100fb_setpalettereg(i, 0, 0, 0, 0, info);
-		sa1100fb_schedule_task(fbi, C_DISABLE);
-		if (sa1100fb_blank_helper)
-			sa1100fb_blank_helper(blank);
-		break;
-
-	case VESA_NO_BLANKING:
-		if (sa1100fb_blank_helper)
-			sa1100fb_blank_helper(blank);
-		if (fbi->fb.disp->visual == FB_VISUAL_PSEUDOCOLOR ||
-		    fbi->fb.disp->visual == FB_VISUAL_STATIC_PSEUDOCOLOR)
-			fb_set_cmap(&fbi->fb.cmap, 1, info);
-		sa1100fb_schedule_task(fbi, C_ENABLE);
-	}
-	return 0;
-}
-
 static int sa1100fb_updatevar(int con, struct fb_info *info)
 {
 	DPRINTK("entered\n");
