@@ -235,6 +235,12 @@
 #define __NR_sched_getaffinity	223
 #define __NR_security		224
 #define __NR_tuxcall		225
+#define __NR_sendfile64		226
+#define __NR_io_setup		227
+#define __NR_io_destroy		228
+#define __NR_io_getevents	229
+#define __NR_io_submit		230
+#define __NR_io_cancel		231
 
 #define __NR(n)	#n
 
