@@ -19,6 +19,7 @@
 #include <asm/io_apic.h>
 #endif
 #include <asm/apic.h>
+#include <asm/thread_info.h>
 #endif
 #endif
 
@@ -36,16 +37,13 @@ extern unsigned long phys_cpu_present_map;
 extern unsigned long cpu_online_map;
 extern volatile unsigned long smp_invalidate_needed;
 extern int pic_mode;
-extern int smp_num_siblings;
-extern int cpu_sibling_map[];
-
 extern void smp_flush_tlb(void);
 extern void smp_message_irq(int cpl, void *dev_id, struct pt_regs *regs);
 extern void smp_send_reschedule(int cpu);
+extern void smp_send_reschedule_all(void);
 extern void smp_invalidate_rcv(void);		/* Process an NMI */
 extern void (*mtrr_hook) (void);
 extern void zap_low_mappings (void);
-extern void smp_stop_cpu(void);
 
 /*
  * On x86 all CPUs are mapped 1:1 to the APIC space.
@@ -83,42 +81,32 @@ extern void smp_store_cpu_info(int id);		/* Store per CPU info (like the initial
 
 #define smp_processor_id() read_pda(cpunumber)
 
-#define stack_smp_processor_id() (stack_current()->processor)
-
-
 extern __inline int hard_smp_processor_id(void)
 {
 	/* we don't want to mark this access volatile - bad code generation */
-	return GET_APIC_ID(*(unsigned *)(APIC_BASE+APIC_ID));
+	return GET_APIC_ID(*(unsigned long *)(APIC_BASE+APIC_ID));
 }
 
-extern int apic_disabled;
-#define safe_smp_processor_id() (apic_disabled ? 0 : x86_apicid_to_cpu[hard_smp_processor_id()])
-
 #endif /* !ASSEMBLY */
 
 #define NO_PROC_ID		0xFF		/* No processor magic marker */
 
-/*
- *	This magic constant controls our willingness to transfer
- *	a process across CPUs. Such a transfer incurs misses on the L1
- *	cache, and on a P6 or P5 with multiple L2 caches L2 hits. My
- *	gut feeling is this will vary by board in value. For a board
- *	with separate L2 cache it probably depends also on the RSS, and
- *	for a board with shared L2 cache it ought to decay fast as other
- *	processes are run.
- */
- 
-#define PROC_CHANGE_PENALTY	15		/* Schedule penalty */
-
-
-
 #endif
 #define INT_DELIVERY_MODE 1     /* logical delivery */
 #define TARGET_CPUS 1
 
+
 #ifndef CONFIG_SMP
 #define stack_smp_processor_id() 0
-#define safe_smp_processor_id() 0
+#else
+#include <asm/thread_info.h>
+#define stack_smp_processor_id() \
+({ 								\
+	struct thread_info *ti;					\
+	__asm__("andq %%rsp,%0; ":"=r" (ti) : "0" (~8191UL));	\
+	ti->cpu;						\
+})
 #endif
+
 #endif
+
