@@ -215,18 +215,12 @@ extern inline char *bio_kmap_irq(struct bio *bio, unsigned long *flags)
 {
 	unsigned long addr;
 
-	__save_flags(*flags);
-
 	/*
-	 * could be low
+	 * might not be a highmem page, but the preempt/irq count
+	 * balancing is a lot nicer this way
 	 */
-	if (!PageHighMem(bio_page(bio)))
-		return bio_data(bio);
-
-	/*
-	 * it's a highmem page
-	 */
-	__cli();
+	local_save_flags(*flags);
+	local_irq_disable();
 	addr = (unsigned long) kmap_atomic(bio_page(bio), KM_BIO_SRC_IRQ);
 
 	if (addr & ~PAGE_MASK)
@@ -240,7 +234,7 @@ extern inline void bio_kunmap_irq(char *buffer, unsigned long *flags)
 	unsigned long ptr = (unsigned long) buffer & PAGE_MASK;
 
 	kunmap_atomic((void *) ptr, KM_BIO_SRC_IRQ);
-	__restore_flags(*flags);
+	local_irq_restore(*flags);
 }
 
 #else
