@@ -1,38 +1,42 @@
 /* SCTP kernel reference Implementation
- * (C) Copyright IBM Corp. 2001, 2004
  * Copyright (c) 1999-2000 Cisco, Inc.
  * Copyright (c) 1999-2001 Motorola, Inc.
  * Copyright (c) 2001 Intel Corp.
- *
+ * Copyright (c) 2001-2002 International Business Machines Corp.
+ * 
  * This file is part of the SCTP kernel reference Implementation
- *
+ * 
+ * This file is part of the implementation of the add-IP extension,
+ * based on <draft-ietf-tsvwg-addip-sctp-02.txt> June 29, 2001,
+ * for the SCTP kernel reference Implementation.
+ * 
  * These are definitions needed by the state machine.
- *
- * The SCTP reference implementation is free software;
- * you can redistribute it and/or modify it under the terms of
+ * 
+ * The SCTP reference implementation is free software; 
+ * you can redistribute it and/or modify it under the terms of 
  * the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *
- * The SCTP reference implementation is distributed in the hope that it
+ * 
+ * The SCTP reference implementation is distributed in the hope that it 
  * will be useful, but WITHOUT ANY WARRANTY; without even the implied
  *                 ************************
  * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with GNU CC; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- *
+ * Boston, MA 02111-1307, USA.  
+ * 
  * Please send any bug reports or fixes you make to the
  * email addresses:
  *    lksctp developers <lksctp-developers@lists.sourceforge.net>
- *
+ * 
  * Or submit a bug report through the following website:
  *    http://www.sf.net/projects/lksctp
  *
- * Written or modified by:
+ * Written or modified by: 
  *    La Monte H.P. Yarroll <piggy@acm.org>
  *    Karl Knutson <karl@athena.chicago.il.us>
  *    Xingang Guo <xingang.guo@intel.com>
@@ -40,15 +44,15 @@
  *    Dajiang Zhang <dajiang.zhang@nokia.com>
  *    Sridhar Samudrala <sri@us.ibm.com>
  *    Daisy Chang <daisyc@us.ibm.com>
- *    Ardelle Fan <ardelle.fan@intel.com>
- *    Kevin Gao <kevin.gao@intel.com>
  *
  * Any bugs reported given to us we will try to fix... any fixes shared will
  * be incorporated into the next SCTP release.
  */
 
+
 #include <linux/types.h>
 #include <linux/compiler.h>
+#include <linux/sched.h>
 #include <linux/slab.h>
 #include <linux/in.h>
 #include <net/sctp/command.h>
@@ -77,15 +81,15 @@ typedef struct {
 	int action;
 } sctp_sm_command_t;
 
-typedef sctp_disposition_t (sctp_state_fn_t) (const struct sctp_endpoint *,
-					      const struct sctp_association *,
+typedef sctp_disposition_t (sctp_state_fn_t) (const sctp_endpoint_t *,
+					      const sctp_association_t *,
 					      const sctp_subtype_t type,
 					      void *arg,
 					      sctp_cmd_seq_t *);
 typedef void (sctp_timer_event_t) (unsigned long);
 typedef struct {
 	sctp_state_fn_t *fn;
-	const char *name;
+	char *name;
 } sctp_sm_table_entry_t;
 
 /* A naming convention of "sctp_sf_xxx" applies to all the state functions
@@ -103,9 +107,6 @@ sctp_state_fn_t sctp_sf_timer_ignore;
 sctp_state_fn_t sctp_sf_do_9_1_abort;
 sctp_state_fn_t sctp_sf_cookie_wait_abort;
 sctp_state_fn_t sctp_sf_cookie_echoed_abort;
-sctp_state_fn_t sctp_sf_shutdown_pending_abort;
-sctp_state_fn_t sctp_sf_shutdown_sent_abort;
-sctp_state_fn_t sctp_sf_shutdown_ack_sent_abort;
 sctp_state_fn_t sctp_sf_do_5_1B_init;
 sctp_state_fn_t sctp_sf_do_5_1C_ack;
 sctp_state_fn_t sctp_sf_do_5_1D_ce;
@@ -118,8 +119,6 @@ sctp_state_fn_t sctp_sf_tabort_8_4_8;
 sctp_state_fn_t sctp_sf_operr_notify;
 sctp_state_fn_t sctp_sf_t1_timer_expire;
 sctp_state_fn_t sctp_sf_t2_timer_expire;
-sctp_state_fn_t sctp_sf_t4_timer_expire;
-sctp_state_fn_t sctp_sf_t5_timer_expire;
 sctp_state_fn_t sctp_sf_sendbeat_8_3;
 sctp_state_fn_t sctp_sf_beat_8_3;
 sctp_state_fn_t sctp_sf_backbeat_8_3;
@@ -128,21 +127,13 @@ sctp_state_fn_t sctp_sf_do_9_2_shutdown;
 sctp_state_fn_t sctp_sf_do_ecn_cwr;
 sctp_state_fn_t sctp_sf_do_ecne;
 sctp_state_fn_t sctp_sf_ootb;
+sctp_state_fn_t sctp_sf_shut_8_4_5;
 sctp_state_fn_t sctp_sf_pdiscard;
 sctp_state_fn_t sctp_sf_violation;
-sctp_state_fn_t sctp_sf_violation_chunklen;
 sctp_state_fn_t sctp_sf_discard_chunk;
 sctp_state_fn_t sctp_sf_do_5_2_1_siminit;
 sctp_state_fn_t sctp_sf_do_5_2_2_dupinit;
 sctp_state_fn_t sctp_sf_do_5_2_4_dupcook;
-sctp_state_fn_t sctp_sf_unk_chunk;
-sctp_state_fn_t sctp_sf_do_8_5_1_E_sa;
-sctp_state_fn_t sctp_sf_cookie_echoed_err;
-sctp_state_fn_t sctp_sf_do_asconf;
-sctp_state_fn_t sctp_sf_do_asconf_ack;
-sctp_state_fn_t sctp_sf_do_9_2_reshutack;
-sctp_state_fn_t sctp_sf_eat_fwd_tsn;
-sctp_state_fn_t sctp_sf_eat_fwd_tsn_fast;
 
 /* Prototypes for primitive event state functions.  */
 sctp_state_fn_t sctp_sf_do_prm_asoc;
@@ -153,144 +144,194 @@ sctp_state_fn_t sctp_sf_cookie_echoed_prm_shutdown;
 sctp_state_fn_t sctp_sf_do_9_1_prm_abort;
 sctp_state_fn_t sctp_sf_cookie_wait_prm_abort;
 sctp_state_fn_t sctp_sf_cookie_echoed_prm_abort;
-sctp_state_fn_t sctp_sf_shutdown_pending_prm_abort;
-sctp_state_fn_t sctp_sf_shutdown_sent_prm_abort;
-sctp_state_fn_t sctp_sf_shutdown_ack_sent_prm_abort;
 sctp_state_fn_t sctp_sf_error_closed;
 sctp_state_fn_t sctp_sf_error_shutdown;
 sctp_state_fn_t sctp_sf_ignore_primitive;
-sctp_state_fn_t sctp_sf_do_prm_requestheartbeat;
-sctp_state_fn_t sctp_sf_do_prm_asconf;
 
 /* Prototypes for other event state functions.  */
 sctp_state_fn_t sctp_sf_do_9_2_start_shutdown;
 sctp_state_fn_t sctp_sf_do_9_2_shutdown_ack;
 sctp_state_fn_t sctp_sf_ignore_other;
-sctp_state_fn_t sctp_sf_cookie_wait_icmp_abort;
 
 /* Prototypes for timeout event state functions.  */
 sctp_state_fn_t sctp_sf_do_6_3_3_rtx;
 sctp_state_fn_t sctp_sf_do_6_2_sack;
 sctp_state_fn_t sctp_sf_autoclose_timer_expire;
 
+
+/* These are state functions which are either obsolete or not in use yet.
+ * If any of these functions needs to be revived, it should be renamed with
+ * the "sctp_sf_xxx" prefix, and be moved to the above prototype groups.
+ */
+
+/* Prototypes for chunk state functions.  Not in use. */
+sctp_state_fn_t sctp_sf_do_5_2_6_stale;
+sctp_state_fn_t sctp_sf_do_9_2_reshutack;
+sctp_state_fn_t sctp_sf_do_9_2_reshut;
+sctp_state_fn_t sctp_sf_do_9_2_shutack;
+
+sctp_state_fn_t lucky;
+sctp_state_fn_t other_stupid;
+
+/* Prototypes for timeout event state functions.  Not in use. */
+sctp_state_fn_t sctp_do_4_2_reinit;
+sctp_state_fn_t sctp_do_4_3_reecho;
+sctp_state_fn_t sctp_do_9_2_reshut;
+sctp_state_fn_t sctp_do_9_2_reshutack;
+sctp_state_fn_t sctp_do_8_3_hb_err;
+sctp_state_fn_t sctp_heartoff;
+
+/* Prototypes for addip related state functions.  Not in use. */
+sctp_state_fn_t sctp_addip_do_asconf;
+sctp_state_fn_t sctp_addip_do_asconf_ack;
+
 /* Prototypes for utility support functions.  */
-__u8 sctp_get_chunk_type(struct sctp_chunk *chunk);
-const sctp_sm_table_entry_t *sctp_sm_lookup_event(sctp_event_t,
-					    sctp_state_t,
-					    sctp_subtype_t);
-int sctp_chunk_iif(const struct sctp_chunk *);
-struct sctp_association *sctp_make_temp_asoc(const struct sctp_endpoint *,
-					     struct sctp_chunk *,
-					     int gfp);
+__u8 sctp_get_chunk_type(sctp_chunk_t *chunk);
+sctp_sm_table_entry_t *sctp_sm_lookup_event(sctp_event_t event_type,
+					    sctp_state_t state,
+					    sctp_subtype_t event_subtype);
+
+time_t timeval_sub(struct timeval *, struct timeval *);
+sctp_association_t *sctp_make_temp_asoc(const sctp_endpoint_t *,
+					sctp_chunk_t *,
+					const int priority);
 __u32 sctp_generate_verification_tag(void);
+sctpParam_t sctp_get_my_addrs_raw(const sctp_association_t *,
+				  const int priority, int *addrs_len);
+
 void sctp_populate_tie_tags(__u8 *cookie, __u32 curTag, __u32 hisTag);
 
 /* Prototypes for chunk-building functions.  */
-struct sctp_chunk *sctp_make_init(const struct sctp_association *,
-			     const struct sctp_bind_addr *,
-			     int gfp, int vparam_len);
-struct sctp_chunk *sctp_make_init_ack(const struct sctp_association *,
-				 const struct sctp_chunk *,
-				 const int gfp,
-				 const int unkparam_len);
-struct sctp_chunk *sctp_make_cookie_echo(const struct sctp_association *,
-				    const struct sctp_chunk *);
-struct sctp_chunk *sctp_make_cookie_ack(const struct sctp_association *,
-				   const struct sctp_chunk *);
-struct sctp_chunk *sctp_make_cwr(const struct sctp_association *,
+sctp_chunk_t *sctp_make_init(const sctp_association_t *,
+			     const sctp_bind_addr_t *,
+			     int priority);
+sctp_chunk_t *sctp_make_init_ack(const sctp_association_t *,
+				 const sctp_chunk_t *,
+				 const int priority);
+sctp_chunk_t *sctp_make_cookie_echo(const sctp_association_t *,
+				    const sctp_chunk_t *);
+sctp_chunk_t *sctp_make_cookie_ack(const sctp_association_t *,
+				   const sctp_chunk_t *);
+sctp_chunk_t *sctp_make_cwr(const sctp_association_t *,
 				 const __u32 lowest_tsn,
-				 const struct sctp_chunk *);
-struct sctp_chunk * sctp_make_datafrag_empty(struct sctp_association *,
+				 const sctp_chunk_t *);
+sctp_chunk_t *sctp_make_datafrag(sctp_association_t *,
+				 const struct sctp_sndrcvinfo *sinfo,
+				 int len, const __u8 *data,
+				 __u8 flags, __u16 ssn);
+sctp_chunk_t * sctp_make_datafrag_empty(sctp_association_t *,
 					const struct sctp_sndrcvinfo *sinfo,
 					int len, const __u8 flags,
 					__u16 ssn);
-struct sctp_chunk *sctp_make_ecne(const struct sctp_association *,
+sctp_chunk_t *sctp_make_data(sctp_association_t *,
+			     const struct sctp_sndrcvinfo *sinfo,
+			     int len, const __u8 *data);
+sctp_chunk_t *sctp_make_data_empty(sctp_association_t *,
+				   const struct sctp_sndrcvinfo *, int len);
+sctp_chunk_t *sctp_make_ecne(const sctp_association_t *,
 				  const __u32);
-struct sctp_chunk *sctp_make_sack(const struct sctp_association *);
-struct sctp_chunk *sctp_make_shutdown(const struct sctp_association *asoc,
-				      const struct sctp_chunk *chunk);
-struct sctp_chunk *sctp_make_shutdown_ack(const struct sctp_association *asoc,
-					  const struct sctp_chunk *);
-struct sctp_chunk *sctp_make_shutdown_complete(const struct sctp_association *,
-					  const struct sctp_chunk *);
-void sctp_init_cause(struct sctp_chunk *, __u16 cause, size_t);
-struct sctp_chunk *sctp_make_abort(const struct sctp_association *,
-			      const struct sctp_chunk *,
+sctp_chunk_t *sctp_make_sack(const sctp_association_t *);
+sctp_chunk_t *sctp_make_shutdown(const sctp_association_t *asoc);
+sctp_chunk_t *sctp_make_shutdown_ack(const sctp_association_t *asoc,
+					  const sctp_chunk_t *);
+sctp_chunk_t *sctp_make_shutdown_complete(const sctp_association_t *,
+					  const sctp_chunk_t *);
+void sctp_init_cause(sctp_chunk_t *, __u16 cause, const void *, size_t);
+sctp_chunk_t *sctp_make_abort(const sctp_association_t *,
+			      const sctp_chunk_t *,
 			      const size_t hint);
-struct sctp_chunk *sctp_make_abort_no_data(const struct sctp_association *,
-				      const struct sctp_chunk *,
+sctp_chunk_t *sctp_make_abort_no_data(const sctp_association_t *,
+				      const sctp_chunk_t *,
 				      __u32 tsn);
-struct sctp_chunk *sctp_make_abort_user(const struct sctp_association *,
-					const struct msghdr *, size_t msg_len);
-struct sctp_chunk *sctp_make_abort_violation(const struct sctp_association *,
-				   const struct sctp_chunk *,
-				   const __u8 *,
-				   const size_t );
-struct sctp_chunk *sctp_make_heartbeat(const struct sctp_association *,
-				  const struct sctp_transport *,
+sctp_chunk_t *sctp_make_heartbeat(const sctp_association_t *,
+				  const sctp_transport_t *,
 				  const void *payload,
 				  const size_t paylen);
-struct sctp_chunk *sctp_make_heartbeat_ack(const struct sctp_association *,
-				      const struct sctp_chunk *,
+sctp_chunk_t *sctp_make_heartbeat_ack(const sctp_association_t *,
+				      const sctp_chunk_t *,
 				      const void *payload,
 				      const size_t paylen);
-struct sctp_chunk *sctp_make_op_error(const struct sctp_association *,
-				 const struct sctp_chunk *chunk,
+sctp_chunk_t *sctp_make_op_error(const sctp_association_t *,
+				 const sctp_chunk_t *chunk,
 				 __u16 cause_code,
 				 const void *payload,
 				 size_t paylen);
+void sctp_chunk_assign_tsn(sctp_chunk_t *);
 
-struct sctp_chunk *sctp_make_asconf_update_ip(struct sctp_association *,
-					      union sctp_addr *,
-					      struct sockaddr *,
-					      int, __u16);
-struct sctp_chunk *sctp_make_asconf_set_prim(struct sctp_association *asoc,
-					     union sctp_addr *addr);
-struct sctp_chunk *sctp_process_asconf(struct sctp_association *asoc,
-				       struct sctp_chunk *asconf);
-int sctp_process_asconf_ack(struct sctp_association *asoc,
-			    struct sctp_chunk *asconf_ack);
-struct sctp_chunk *sctp_make_fwdtsn(const struct sctp_association *asoc,
-				    __u32 new_cum_tsn, size_t nstreams,
-				    struct sctp_fwdtsn_skip *skiplist);
-
-void sctp_chunk_assign_tsn(struct sctp_chunk *);
-void sctp_chunk_assign_ssn(struct sctp_chunk *);
-
-void sctp_stop_t1_and_abort(sctp_cmd_seq_t *commands, __u16 error);
 
 /* Prototypes for statetable processing. */
 
 int sctp_do_sm(sctp_event_t event_type, sctp_subtype_t subtype,
 	       sctp_state_t state,
-               struct sctp_endpoint *,
-               struct sctp_association *asoc,
+               sctp_endpoint_t *,
+               sctp_association_t *asoc,
                void *event_arg,
-               int gfp);
+               int priority);
+
+int sctp_side_effects(sctp_event_t event_type, sctp_subtype_t subtype,
+		      sctp_state_t state,
+                      sctp_endpoint_t *,
+                      sctp_association_t *asoc,
+                      void *event_arg,
+                      sctp_disposition_t status,
+		      sctp_cmd_seq_t *commands,
+                      int priority);
 
 /* 2nd level prototypes */
+int
+sctp_cmd_interpreter(sctp_event_t event_type, sctp_subtype_t subtype,
+		     sctp_state_t state,
+		     sctp_endpoint_t *ep,
+		     sctp_association_t *asoc,
+		     void *event_arg,
+		     sctp_disposition_t status,
+		     sctp_cmd_seq_t *retval,
+		     int priority);
+
+
+int sctp_gen_sack(sctp_association_t *, int force, sctp_cmd_seq_t *);
+void sctp_do_TSNdup(sctp_association_t *, sctp_chunk_t *, long gap);
+
 void sctp_generate_t3_rtx_event(unsigned long peer);
 void sctp_generate_heartbeat_event(unsigned long peer);
 
-void sctp_ootb_pkt_free(struct sctp_packet *);
+sctp_sackhdr_t *sctp_sm_pull_sack(sctp_chunk_t *);
 
-struct sctp_association *sctp_unpack_cookie(const struct sctp_endpoint *,
-				       const struct sctp_association *,
-				       struct sctp_chunk *, int gfp, int *err,
-				       struct sctp_chunk **err_chk_p);
-int sctp_addip_addr_config(struct sctp_association *, sctp_param_t,
+sctp_cookie_param_t *
+sctp_pack_cookie(const sctp_endpoint_t *, const sctp_association_t *,
+		 const sctp_chunk_t *, int *cookie_len,
+		 const __u8 *, int addrs_len);
+sctp_association_t *sctp_unpack_cookie(const sctp_endpoint_t *,
+				       const sctp_association_t *,
+				       sctp_chunk_t *, int priority, int *err);
+int sctp_addip_addr_config(sctp_association_t *, sctp_param_t,
 			   struct sockaddr_storage*, int);
 
 /* 3rd level prototypes */
-__u32 sctp_generate_tag(const struct sctp_endpoint *);
-__u32 sctp_generate_tsn(const struct sctp_endpoint *);
+__u32 sctp_generate_tag(const sctp_endpoint_t *);
+__u32 sctp_generate_tsn(const sctp_endpoint_t *);
+
+/* 4th level prototypes */
+void sctp_param2sockaddr(sockaddr_storage_t *addr, const sctpParam_t param,
+			 __u16 port);
+int sctp_addr2sockaddr(const sctpParam_t, sockaddr_storage_t *);
+int sockaddr2sctp_addr(const sockaddr_storage_t *, sctpParam_t);
 
 /* Extern declarations for major data structures.  */
+sctp_sm_table_entry_t *sctp_chunk_event_lookup(sctp_cid_t, sctp_state_t);
+extern sctp_sm_table_entry_t
+primitive_event_table[SCTP_NUM_PRIMITIVE_TYPES][SCTP_STATE_NUM_STATES];
+extern sctp_sm_table_entry_t
+other_event_table[SCTP_NUM_OTHER_TYPES][SCTP_STATE_NUM_STATES];
+extern sctp_sm_table_entry_t
+timeout_event_table[SCTP_NUM_TIMEOUT_TYPES][SCTP_STATE_NUM_STATES];
 extern sctp_timer_event_t *sctp_timer_events[SCTP_NUM_TIMEOUT_TYPES];
 
+/* These are some handy utility macros... */
+
 
 /* Get the size of a DATA chunk payload. */
-static inline __u16 sctp_data_size(struct sctp_chunk *chunk)
+static inline __u16 sctp_data_size(sctp_chunk_t *chunk)
 {
 	__u16 size;
 
@@ -367,21 +408,6 @@ static inline int SSN_lte(__u16 s, __u16 t)
 	return (((s) == (t)) || (((s) - (t)) & SSN_SIGN_BIT));
 }
 
-/*
- * ADDIP 3.1.1
- * The valid range of Serial Number is from 0 to 4294967295 (2**32 - 1). Serial
- * Numbers wrap back to 0 after reaching 4294967295.
- */
-enum {
-	ADDIP_SERIAL_SIGN_BIT = (1<<31)
-};
-
-static inline int ADDIP_SERIAL_gte(__u16 s, __u16 t)
-{
-	return (((s) == (t)) || (((t) - (s)) & ADDIP_SERIAL_SIGN_BIT));
-}
-
-
 /* Run sctp_add_cmd() generating a BUG() if there is a failure.  */
 static inline void sctp_add_cmd_sf(sctp_cmd_seq_t *seq, sctp_verb_t verb, sctp_arg_t obj)
 {
@@ -389,53 +415,4 @@ static inline void sctp_add_cmd_sf(sctp_cmd_seq_t *seq, sctp_verb_t verb, sctp_a
 		BUG();
 }
 
-/* Check VTAG of the packet matches the sender's own tag. */
-static inline int
-sctp_vtag_verify(const struct sctp_chunk *chunk,
-		 const struct sctp_association *asoc)
-{
-	/* RFC 2960 Sec 8.5 When receiving an SCTP packet, the endpoint
-	 * MUST ensure that the value in the Verification Tag field of
-	 * the received SCTP packet matches its own Tag. If the received
-	 * Verification Tag value does not match the receiver's own
-	 * tag value, the receiver shall silently discard the packet...
-	 */
-        if (ntohl(chunk->sctp_hdr->vtag) == asoc->c.my_vtag)
-                return 1;
-
-	return 0;
-}
-
-/* Check VTAG of the packet matches the sender's own tag OR its peer's
- * tag and the T bit is set in the Chunk Flags.
- */
-static inline int
-sctp_vtag_verify_either(const struct sctp_chunk *chunk,
-			const struct sctp_association *asoc)
-{
-        /* RFC 2960 Section 8.5.1, sctpimpguide-06 Section 2.13.2
-	 *
-	 * B) The receiver of a ABORT shall accept the packet if the
-	 * Verification Tag field of the packet matches its own tag OR it
-	 * is set to its peer's tag and the T bit is set in the Chunk
-	 * Flags. Otherwise, the receiver MUST silently discard the packet
-	 * and take no further action.
-	 *
-	 * (C) The receiver of a SHUTDOWN COMPLETE shall accept the
-	 * packet if the Verification Tag field of the packet
-	 * matches its own tag OR it is set to its peer's tag and
-	 * the T bit is set in the Chunk Flags.  Otherwise, the
-	 * receiver MUST silently discard the packet and take no
-	 * further action....
-	 *
-	 */
-        if ((ntohl(chunk->sctp_hdr->vtag) == asoc->c.my_vtag) ||
-	    (sctp_test_T_bit(chunk) && (ntohl(chunk->sctp_hdr->vtag)
-	    == asoc->c.peer_vtag))) {
-                return 1;
-	}
-
-	return 0;
-}
-
 #endif /* __sctp_sm_h__ */
