@@ -62,18 +62,18 @@
 #include <net/ip_fib.h>
 
 struct ipv4_devconf ipv4_devconf = {
-	accept_redirects: 1,
-	send_redirects:	  1,
-	secure_redirects: 1,
-	shared_media:	  1,
+	.accept_redirects = 1,
+	.send_redirects =  1,
+	.secure_redirects = 1,
+	.shared_media =	  1,
 };
 
 static struct ipv4_devconf ipv4_devconf_dflt = {
-	accept_redirects:    1,
-	send_redirects:      1,
-	secure_redirects:    1,
-	shared_media:	     1,
-	accept_source_route: 1,
+	.accept_redirects =  1,
+	.send_redirects =    1,
+	.secure_redirects =  1,
+	.shared_media =	     1,
+	.accept_source_route = 1,
 };
 
 static void rtmsg_ifa(int event, struct in_ifaddr *);
@@ -886,7 +886,7 @@ out:
 }
 
 struct notifier_block ip_netdev_notifier = {
-	notifier_call:	inetdev_event,
+	.notifier_call =inetdev_event,
 };
 
 static int inet_fill_ifaddr(struct sk_buff *skb, struct in_ifaddr *ifa,
@@ -1047,150 +1047,150 @@ static struct devinet_sysctl_table {
 	ctl_table		devinet_proto_dir[2];
 	ctl_table		devinet_root_dir[2];
 } devinet_sysctl = {
-	devinet_vars: {
+	.devinet_vars = {
 		{
-			ctl_name:	NET_IPV4_CONF_FORWARDING,
-			procname:	"forwarding",
-			data:		&ipv4_devconf.forwarding,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&devinet_sysctl_forward,
+			.ctl_name =	NET_IPV4_CONF_FORWARDING,
+			.procname =	"forwarding",
+			.data =	&ipv4_devconf.forwarding,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&devinet_sysctl_forward,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_MC_FORWARDING,
-			procname:	"mc_forwarding",
-			data:		&ipv4_devconf.mc_forwarding,
-			maxlen:		sizeof(int),
-			mode:		0444,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_MC_FORWARDING,
+			.procname =	"mc_forwarding",
+			.data =	&ipv4_devconf.mc_forwarding,
+			.maxlen =		sizeof(int),
+			.mode =	0444,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_ACCEPT_REDIRECTS,
-			procname:	"accept_redirects",
-			data:		&ipv4_devconf.accept_redirects,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_ACCEPT_REDIRECTS,
+			.procname =	"accept_redirects",
+			.data =	&ipv4_devconf.accept_redirects,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_SECURE_REDIRECTS,
-			procname:	"secure_redirects",
-			data:		&ipv4_devconf.secure_redirects,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_SECURE_REDIRECTS,
+			.procname =	"secure_redirects",
+			.data =	&ipv4_devconf.secure_redirects,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_SHARED_MEDIA,
-			procname:	"shared_media",
-			data:		&ipv4_devconf.shared_media,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_SHARED_MEDIA,
+			.procname =	"shared_media",
+			.data =	&ipv4_devconf.shared_media,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_RP_FILTER,
-			procname:	"rp_filter",
-			data:		&ipv4_devconf.rp_filter,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_RP_FILTER,
+			.procname =	"rp_filter",
+			.data =	&ipv4_devconf.rp_filter,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_SEND_REDIRECTS,
-			procname:	"send_redirects",
-			data:		&ipv4_devconf.send_redirects,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_SEND_REDIRECTS,
+			.procname =	"send_redirects",
+			.data =	&ipv4_devconf.send_redirects,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_ACCEPT_SOURCE_ROUTE,
-			procname:	"accept_source_route",
-			data:		&ipv4_devconf.accept_source_route,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_ACCEPT_SOURCE_ROUTE,
+			.procname =	"accept_source_route",
+			.data =	&ipv4_devconf.accept_source_route,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_PROXY_ARP,
-			procname:	"proxy_arp",
-			data:		&ipv4_devconf.proxy_arp,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_PROXY_ARP,
+			.procname =	"proxy_arp",
+			.data =	&ipv4_devconf.proxy_arp,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_MEDIUM_ID,
-			procname:	"medium_id",
-			data:		&ipv4_devconf.medium_id,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_MEDIUM_ID,
+			.procname =	"medium_id",
+			.data =	&ipv4_devconf.medium_id,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_BOOTP_RELAY,
-			procname:	"bootp_relay",
-			data:		&ipv4_devconf.bootp_relay,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_BOOTP_RELAY,
+			.procname =	"bootp_relay",
+			.data =	&ipv4_devconf.bootp_relay,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_LOG_MARTIANS,
-			procname:	"log_martians",
-			data:		&ipv4_devconf.log_martians,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_LOG_MARTIANS,
+			.procname =	"log_martians",
+			.data =	&ipv4_devconf.log_martians,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_TAG,
-			procname:	"tag",
-			data:		&ipv4_devconf.tag,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_TAG,
+			.procname =	"tag",
+			.data =	&ipv4_devconf.tag,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 		{
-			ctl_name:	NET_IPV4_CONF_ARPFILTER,
-			procname:	"arp_filter",
-			data:		&ipv4_devconf.arp_filter,
-			maxlen:		sizeof(int),
-			mode:		0644,
-			proc_handler:	&proc_dointvec,
+			.ctl_name =	NET_IPV4_CONF_ARPFILTER,
+			.procname =	"arp_filter",
+			.data =	&ipv4_devconf.arp_filter,
+			.maxlen =		sizeof(int),
+			.mode =	0644,
+			.proc_handler =&proc_dointvec,
 		},
 	},
-	devinet_dev: {
+	.devinet_dev = {
 		{
-			ctl_name:	NET_PROTO_CONF_ALL,
-			procname:	"all",
-			mode:		0555,
-			child:		devinet_sysctl.devinet_vars,
+			.ctl_name =	NET_PROTO_CONF_ALL,
+			.procname =	"all",
+			.mode =	0555,
+			.child =		devinet_sysctl.devinet_vars,
 		},
 	},
-	devinet_conf_dir: {
+	.devinet_conf_dir = {
 	        {
-			ctl_name:	NET_IPV4_CONF,
-			procname:	"conf",
-			mode:		0555,
-			child:		devinet_sysctl.devinet_dev,
+			.ctl_name =	NET_IPV4_CONF,
+			.procname =	"conf",
+			.mode =	0555,
+			.child =		devinet_sysctl.devinet_dev,
 		},
 	},
-	devinet_proto_dir: {
+	.devinet_proto_dir = {
 		{
-			ctl_name:	NET_IPV4,
-			procname:	"ipv4",
-			mode:		0555,
-			child:		devinet_sysctl.devinet_conf_dir,
+			.ctl_name =	NET_IPV4,
+			.procname =	"ipv4",
+			.mode =	0555,
+			.child =		devinet_sysctl.devinet_conf_dir,
 		},
 	},
-	devinet_root_dir: {
+	.devinet_root_dir = {
 		{
-			ctl_name:	CTL_NET,
-			procname:	"net",
-			mode:		0555,
-			child:		devinet_sysctl.devinet_proto_dir,
+			.ctl_name =	CTL_NET,
+			.procname =	"net",
+			.mode =	0555,
+			.child =		devinet_sysctl.devinet_proto_dir,
 		},
 	},
 };
