@@ -1,43 +1,41 @@
 /* SCTP kernel reference Implementation
- * (C) Copyright IBM Corp. 2001, 2004
  * Copyright (c) 1999-2000 Cisco, Inc.
  * Copyright (c) 1999-2001 Motorola, Inc.
+ * Copyright (c) 2001 International Business Machines, Corp.
  * Copyright (c) 2001 Intel Corp.
  * Copyright (c) 2001 Nokia, Inc.
  * Copyright (c) 2001 La Monte H.P. Yarroll
- *
- * These functions manipulate an sctp event.   The struct ulpevent is used
- * to carry notifications and data to the ULP (sockets).
- * The SCTP reference implementation is free software;
- * you can redistribute it and/or modify it under the terms of
+ * 
+ * These functions manipulate an sctp event.   The sctp_ulpevent_t is used
+ * to carry notifications and data to the ULP (sockets).  
+ * The SCTP reference implementation is free software; 
+ * you can redistribute it and/or modify it under the terms of 
  * the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *
- * The SCTP reference implementation is distributed in the hope that it
+ * 
+ * The SCTP reference implementation is distributed in the hope that it 
  * will be useful, but WITHOUT ANY WARRANTY; without even the implied
  *                 ************************
  * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with GNU CC; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- *
+ * Boston, MA 02111-1307, USA.  
+ * 
  * Please send any bug reports or fixes you make to the
  * email address(es):
  *    lksctp developers <lksctp-developers@lists.sourceforge.net>
- *
+ * 
  * Or submit a bug report through the following website:
  *    http://www.sf.net/projects/lksctp
  *
- * Written or modified by:
+ * Written or modified by: 
  *    Jon Grimm             <jgrimm@us.ibm.com>
  *    La Monte H.P. Yarroll <piggy@acm.org>
- *    Ardelle Fan	    <ardelle.fan@intel.com>
- *    Sridhar Samudrala     <sri@us.ibm.com>
- *
+ * 
  * Any bugs reported given to us we will try to fix... any fixes shared will
  * be incorporated into the next SCTP release.
  */
@@ -48,78 +46,58 @@
 #include <net/sctp/sctp.h>
 #include <net/sctp/sm.h>
 
-static void sctp_ulpevent_receive_data(struct sctp_ulpevent *event,
-				       struct sctp_association *asoc);
-static void sctp_ulpevent_release_data(struct sctp_ulpevent *event);
-
-/* Stub skb destructor.  */
-static void sctp_stub_rfree(struct sk_buff *skb)
-{
-/* WARNING:  This function is just a warning not to use the
- * skb destructor.  If the skb is shared, we may get the destructor
- * callback on some processor that does not own the sock_lock.  This
- * was occuring with PACKET socket applications that were monitoring
- * our skbs.   We can't take the sock_lock, because we can't risk
- * recursing if we do really own the sock lock.  Instead, do all
- * of our rwnd manipulation while we own the sock_lock outright.
- */
-}
-
-/* Initialize an ULP event from an given skb.  */
-SCTP_STATIC void sctp_ulpevent_init(struct sctp_ulpevent *event, int msg_flags)
-{
-	memset(event, 0, sizeof(struct sctp_ulpevent));
-	event->msg_flags = msg_flags;
-}
+static void sctp_rcvmsg_rfree(struct sk_buff *skb);
+static void sctp_ulpevent_set_owner_r(struct sk_buff *skb,
+				      sctp_association_t *asoc);
 
 /* Create a new sctp_ulpevent.  */
-SCTP_STATIC struct sctp_ulpevent *sctp_ulpevent_new(int size, int msg_flags,
-						    int gfp)
+sctp_ulpevent_t *sctp_ulpevent_new(int size, int msg_flags, int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sk_buff *skb;
 
-	skb = alloc_skb(size, gfp);
+	skb = alloc_skb(size, priority);
 	if (!skb)
 		goto fail;
 
-	event = sctp_skb2event(skb);
-	sctp_ulpevent_init(event, msg_flags);
+	event = (sctp_ulpevent_t *) skb->cb;
+	event = sctp_ulpevent_init(event, skb, msg_flags);
+	if (!event)
+		goto fail_init;
 
+	event->malloced = 1;
 	return event;
 
+fail_init:
+	kfree_skb(event->parent);
+
 fail:
 	return NULL;
 }
 
+/* Initialize an ULP event from an given skb.  */
+sctp_ulpevent_t *sctp_ulpevent_init(sctp_ulpevent_t *event,
+				    struct sk_buff *parent,
+				    int msg_flags)
+{
+	memset(event, sizeof(sctp_ulpevent_t), 0x00);
+	event->msg_flags = msg_flags;
+	event->parent = parent;
+	event->malloced = 0;
+	return event;
+}
+
+/* Dispose of an event.  */
+void sctp_ulpevent_free(sctp_ulpevent_t *event)
+{
+	if (event->malloced)
+		kfree_skb(event->parent);
+}
+
 /* Is this a MSG_NOTIFICATION?  */
-int sctp_ulpevent_is_notification(const struct sctp_ulpevent *event)
+int sctp_ulpevent_is_notification(const sctp_ulpevent_t *event)
 {
-	return MSG_NOTIFICATION == (event->msg_flags & MSG_NOTIFICATION);
-}
-
-/* Hold the association in case the msg_name needs read out of
- * the association.
- */
-static inline void sctp_ulpevent_set_owner(struct sctp_ulpevent *event,
-					   const struct sctp_association *asoc)
-{
-	struct sk_buff *skb;
-
-	/* Cast away the const, as we are just wanting to
-	 * bump the reference count.
-	 */
-	sctp_association_hold((struct sctp_association *)asoc);
-	skb = sctp_event2skb(event);
-	skb->sk = asoc->base.sk;
-	event->asoc = (struct sctp_association *)asoc;
-	skb->destructor = sctp_stub_rfree;
-}
-
-/* A simple destructor to give up the reference to the association. */
-static inline void sctp_ulpevent_release_owner(struct sctp_ulpevent *event)
-{
-	sctp_association_put(event->asoc);
+	return event->msg_flags & MSG_NOTIFICATION;
 }
 
 /* Create and initialize an SCTP_ASSOC_CHANGE event.
@@ -133,22 +111,26 @@ static inline void sctp_ulpevent_release_owner(struct sctp_ulpevent *event)
  * Note: There is no field checking here.  If a field is unused it will be
  * zero'd out.
  */
-struct sctp_ulpevent  *sctp_ulpevent_make_assoc_change(
-	const struct sctp_association *asoc,
-	__u16 flags, __u16 state, __u16 error, __u16 outbound,
-	__u16 inbound, int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_assoc_change(const sctp_association_t *asoc,
+						 __u16 flags,
+						 __u16 state,
+						 __u16 error,
+						 __u16 outbound,
+						 __u16 inbound,
+						 int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sctp_assoc_change *sac;
-	struct sk_buff *skb;
 
 	event = sctp_ulpevent_new(sizeof(struct sctp_assoc_change),
-				  MSG_NOTIFICATION, gfp);
+				  MSG_NOTIFICATION,
+				  priority);
 	if (!event)
 		goto fail;
-	skb = sctp_event2skb(event);
+
 	sac = (struct sctp_assoc_change *)
-		skb_put(skb, sizeof(struct sctp_assoc_change));
+		skb_put(event->parent,
+			sizeof(struct sctp_assoc_change));
 
 	/* Socket Extensions for SCTP
 	 * 5.3.1.1 SCTP_ASSOC_CHANGE
@@ -217,13 +199,12 @@ struct sctp_ulpevent  *sctp_ulpevent_make_assoc_change(
 	 * All notifications for a given association have the same association
 	 * identifier.  For TCP style socket, this field is ignored.
 	 */
-	sctp_ulpevent_set_owner(event, asoc);
 	sac->sac_assoc_id = sctp_assoc2id(asoc);
 
 	return event;
 
 fail:
-	return NULL;
+	return NULL;	
 }
 
 /* Create and initialize an SCTP_PEER_ADDR_CHANGE event.
@@ -234,23 +215,26 @@ fail:
  * When a destination address on a multi-homed peer encounters a change
  * an interface details event is sent.
  */
-struct sctp_ulpevent *sctp_ulpevent_make_peer_addr_change(
-	const struct sctp_association *asoc,
-	const struct sockaddr_storage *aaddr,
-	int flags, int state, int error, int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_peer_addr_change(
+				const sctp_association_t *asoc,
+				const struct sockaddr_storage *aaddr,
+				int flags,
+				int state,
+				int error,
+				int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sctp_paddr_change  *spc;
-	struct sk_buff *skb;
 
 	event = sctp_ulpevent_new(sizeof(struct sctp_paddr_change),
-				  MSG_NOTIFICATION, gfp);
+				  MSG_NOTIFICATION,
+				  priority);
 	if (!event)
 		goto fail;
 
-	skb = sctp_event2skb(event);
 	spc = (struct sctp_paddr_change *)
-		skb_put(skb, sizeof(struct sctp_paddr_change));
+		skb_put(event->parent,
+			sizeof(struct sctp_paddr_change));
 
 	/* Sockets API Extensions for SCTP
 	 * Section 5.3.1.2 SCTP_PEER_ADDR_CHANGE
@@ -283,7 +267,7 @@ struct sctp_ulpevent *sctp_ulpevent_make_peer_addr_change(
 	 * Section 5.3.1.2 SCTP_PEER_ADDR_CHANGE
 	 *
 	 * spc_state:  32 bits (signed integer)
-	 *
+	 * 
 	 * This field holds one of a number of values that communicate the
 	 * event that happened to the address.
 	 */
@@ -303,13 +287,12 @@ struct sctp_ulpevent *sctp_ulpevent_make_peer_addr_change(
 	/* Socket Extensions for SCTP
 	 * 5.3.1.1 SCTP_ASSOC_CHANGE
 	 *
-	 * spc_assoc_id: sizeof (sctp_assoc_t)
+	 * sac_assoc_id: sizeof (sctp_assoc_t)
 	 *
 	 * The association id field, holds the identifier for the association.
 	 * All notifications for a given association have the same association
 	 * identifier.  For TCP style socket, this field is ignored.
 	 */
-	sctp_ulpevent_set_owner(event, asoc);
 	spc->spc_assoc_id = sctp_assoc2id(asoc);
 
 	/* Sockets API Extensions for SCTP
@@ -322,11 +305,6 @@ struct sctp_ulpevent *sctp_ulpevent_make_peer_addr_change(
 	 */
 	memcpy(&spc->spc_aaddr, aaddr, sizeof(struct sockaddr_storage));
 
-	/* Map ipv4 address into v4-mapped-on-v6 address.  */
-	sctp_get_pf_specific(asoc->base.sk->family)->addr_v4map(
-					sctp_sk(asoc->base.sk),
-					(union sctp_addr *)&spc->spc_aaddr);
-
 	return event;
 
 fail:
@@ -348,11 +326,12 @@ fail:
  * specification [SCTP] and any extensions for a list of possible
  * error formats.
  */
-struct sctp_ulpevent *sctp_ulpevent_make_remote_error(
-	const struct sctp_association *asoc, struct sctp_chunk *chunk,
-	__u16 flags, int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_remote_error(const sctp_association_t *asoc,
+						 sctp_chunk_t *chunk,
+						 __u16 flags,
+						 int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sctp_remote_error *sre;
 	struct sk_buff *skb;
 	sctp_errhdr_t *ch;
@@ -361,7 +340,7 @@ struct sctp_ulpevent *sctp_ulpevent_make_remote_error(
 
 	ch = (sctp_errhdr_t *)(chunk->skb->data);
 	cause = ch->cause;
-	elen = WORD_ROUND(ntohs(ch->length)) - sizeof(sctp_errhdr_t);
+	elen = ntohs(ch->length) - sizeof(sctp_errhdr_t);
 
 	/* Pull off the ERROR header.  */
 	skb_pull(chunk->skb, sizeof(sctp_errhdr_t));
@@ -369,8 +348,10 @@ struct sctp_ulpevent *sctp_ulpevent_make_remote_error(
 	/* Copy the skb to a new skb with room for us to prepend
 	 * notification with.
 	 */
-	skb = skb_copy_expand(chunk->skb, sizeof(struct sctp_remote_error),
-			      0, gfp);
+	skb = skb_copy_expand(chunk->skb,
+			      sizeof(struct sctp_remote_error), /* headroom */
+			      0,                                /* tailroom */
+			      priority);
 
 	/* Pull off the rest of the cause TLV from the chunk.  */
 	skb_pull(chunk->skb, elen);
@@ -378,9 +359,14 @@ struct sctp_ulpevent *sctp_ulpevent_make_remote_error(
 		goto fail;
 
 	/* Embed the event fields inside the cloned skb.  */
-	event = sctp_skb2event(skb);
-	sctp_ulpevent_init(event, MSG_NOTIFICATION);
+	event = (sctp_ulpevent_t *) skb->cb;
+	event = sctp_ulpevent_init(event,
+				   skb,
+				   MSG_NOTIFICATION);
+	if (!event)
+		goto fail;
 
+	event->malloced = 1;
 	sre = (struct sctp_remote_error *)
 		skb_push(skb, sizeof(struct sctp_remote_error));
 
@@ -432,7 +418,6 @@ struct sctp_ulpevent *sctp_ulpevent_make_remote_error(
 	 * All notifications for a given association have the same association
 	 * identifier.  For TCP style socket, this field is ignored.
 	 */
-	sctp_ulpevent_set_owner(event, asoc);
 	sre->sre_assoc_id = sctp_assoc2id(asoc);
 
 	return event;
@@ -446,32 +431,37 @@ fail:
  * Socket Extensions for SCTP - draft-01
  * 5.3.1.4 SCTP_SEND_FAILED
  */
-struct sctp_ulpevent *sctp_ulpevent_make_send_failed(
-	const struct sctp_association *asoc, struct sctp_chunk *chunk,
-	__u16 flags, __u32 error, int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_send_failed(const sctp_association_t *asoc,
+						sctp_chunk_t *chunk,
+						__u16 flags,
+						__u32 error,
+						int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sctp_send_failed *ssf;
 	struct sk_buff *skb;
 
-	/* Pull off any padding. */
-	int len = ntohs(chunk->chunk_hdr->length);
-
 	/* Make skb with more room so we can prepend notification.  */
 	skb = skb_copy_expand(chunk->skb,
 			      sizeof(struct sctp_send_failed), /* headroom */
 			      0,                               /* tailroom */
-			      gfp);
+			      priority);
 	if (!skb)
 		goto fail;
 
 	/* Pull off the common chunk header and DATA header.  */
-	skb_pull(skb, sizeof(struct sctp_data_chunk));
-	len -= sizeof(struct sctp_data_chunk);
+	skb_pull(skb, sizeof(sctp_data_chunk_t));
 
 	/* Embed the event fields inside the cloned skb.  */
-	event = sctp_skb2event(skb);
-	sctp_ulpevent_init(event, MSG_NOTIFICATION);
+	event = (sctp_ulpevent_t *) skb->cb;
+	event = sctp_ulpevent_init(event, skb, MSG_NOTIFICATION);
+	if (!event)
+		goto fail;
+
+	/* Mark as malloced, even though the constructor was not
+	 * called.
+	 */
+	event->malloced = 1;
 
 	ssf = (struct sctp_send_failed *)
 		skb_push(skb, sizeof(struct sctp_send_failed));
@@ -506,8 +496,7 @@ struct sctp_ulpevent *sctp_ulpevent_make_send_failed(
 	 * This field is the total length of the notification data, including
 	 * the notification header.
 	 */
-	ssf->ssf_length = sizeof(struct sctp_send_failed) + len;
-	skb_trim(skb, ssf->ssf_length);
+	ssf->ssf_length = skb->len;
 
 	/* Socket Extensions for SCTP
 	 * 5.3.1.4 SCTP_SEND_FAILED
@@ -526,12 +515,9 @@ struct sctp_ulpevent *sctp_ulpevent_make_send_failed(
 	 * The original send information associated with the undelivered
 	 * message.
 	 */
-	memcpy(&ssf->ssf_info, &chunk->sinfo, sizeof(struct sctp_sndrcvinfo));
-
-	/* Per TSVWG discussion with Randy. Allow the application to
-	 * ressemble a fragmented message.
-	 */
-	ssf->ssf_info.sinfo_flags = chunk->chunk_hdr->flags;
+	memcpy(&ssf->ssf_info,
+	       &chunk->sinfo,
+	       sizeof(struct sctp_sndrcvinfo));
 
 	/* Socket Extensions for SCTP
 	 * 5.3.1.4 SCTP_SEND_FAILED
@@ -542,8 +528,8 @@ struct sctp_ulpevent *sctp_ulpevent_make_send_failed(
 	 * same association identifier.  For TCP style socket, this field is
 	 * ignored.
 	 */
-	sctp_ulpevent_set_owner(event, asoc);
 	ssf->ssf_assoc_id = sctp_assoc2id(asoc);
+
 	return event;
 
 fail:
@@ -555,22 +541,22 @@ fail:
  * Socket Extensions for SCTP - draft-01
  * 5.3.1.5 SCTP_SHUTDOWN_EVENT
  */
-struct sctp_ulpevent *sctp_ulpevent_make_shutdown_event(
-	const struct sctp_association *asoc,
-	__u16 flags, int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_shutdown_event(const sctp_association_t *asoc,
+						   __u16 flags,
+						   int priority)
 {
-	struct sctp_ulpevent *event;
+	sctp_ulpevent_t *event;
 	struct sctp_shutdown_event *sse;
-	struct sk_buff *skb;
 
-	event = sctp_ulpevent_new(sizeof(struct sctp_shutdown_event),
-				  MSG_NOTIFICATION, gfp);
+	event = sctp_ulpevent_new(sizeof(struct sctp_assoc_change),
+				  MSG_NOTIFICATION,
+				  priority);
 	if (!event)
 		goto fail;
 
-	skb = sctp_event2skb(event);
 	sse = (struct sctp_shutdown_event *)
-		skb_put(skb, sizeof(struct sctp_shutdown_event));
+		skb_put(event->parent,
+			sizeof(struct sctp_shutdown_event));
 
 	/* Socket Extensions for SCTP
 	 * 5.3.1.5 SCTP_SHUTDOWN_EVENT
@@ -605,7 +591,6 @@ struct sctp_ulpevent *sctp_ulpevent_make_shutdown_event(
 	 * All notifications for a given association have the same association
 	 * identifier.  For TCP style socket, this field is ignored.
 	 */
-	sctp_ulpevent_set_owner(event, asoc);
 	sse->sse_assoc_id = sctp_assoc2id(asoc);
 
 	return event;
@@ -614,60 +599,27 @@ fail:
 	return NULL;
 }
 
-/* Create and initialize a SCTP_ADAPTION_INDICATION notification.
- *
- * Socket Extensions for SCTP
- * 5.3.1.6 SCTP_ADAPTION_INDICATION
- */
-struct sctp_ulpevent *sctp_ulpevent_make_adaption_indication(
-	const struct sctp_association *asoc, int gfp)
-{
-	struct sctp_ulpevent *event;
-	struct sctp_adaption_event *sai;
-	struct sk_buff *skb;
-
-	event = sctp_ulpevent_new(sizeof(struct sctp_adaption_event),
-				  MSG_NOTIFICATION, gfp);
-	if (!event)
-		goto fail;
-
-	skb = sctp_event2skb(event);
-	sai = (struct sctp_adaption_event *)
-		skb_put(skb, sizeof(struct sctp_adaption_event));
-
-	sai->sai_type = SCTP_ADAPTION_INDICATION;
-	sai->sai_flags = 0;
-	sai->sai_length = sizeof(struct sctp_adaption_event);
-	sai->sai_adaption_ind = asoc->peer.adaption_ind;
-	sctp_ulpevent_set_owner(event, asoc);
-	sai->sai_assoc_id = sctp_assoc2id(asoc);
-
-	return event;
-
-fail:
-	return NULL;
-}
-
 /* A message has been received.  Package this message as a notification
  * to pass it to the upper layers.  Go ahead and calculate the sndrcvinfo
  * even if filtered out later.
  *
- * Socket Extensions for SCTP
+ * Socket Extensions for SCTP - draft-01
  * 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
  */
-struct sctp_ulpevent *sctp_ulpevent_make_rcvmsg(struct sctp_association *asoc,
-						struct sctp_chunk *chunk,
-						int gfp)
+sctp_ulpevent_t *sctp_ulpevent_make_rcvmsg(sctp_association_t *asoc,
+					   sctp_chunk_t *chunk,
+					   int priority)
 {
-	struct sctp_ulpevent *event = NULL;
+	sctp_ulpevent_t *event;
+	struct sctp_sndrcvinfo *info;
 	struct sk_buff *skb;
 	size_t padding, len;
 
 	/* Clone the original skb, sharing the data.  */
-	skb = skb_clone(chunk->skb, gfp);
+	skb = skb_clone(chunk->skb, priority);
 	if (!skb)
 		goto fail;
-
+		
 	/* First calculate the padding, so we don't inadvertently
 	 * pass up the wrong length to the user.
 	 *
@@ -686,128 +638,47 @@ struct sctp_ulpevent *sctp_ulpevent_make_rcvmsg(struct sctp_association *asoc,
 	/* Fixup cloned skb with just this chunks data.  */
 	skb_trim(skb, chunk->chunk_end - padding - skb->data);
 
+	/* Set up a destructor to do rwnd accounting.  */
+	sctp_ulpevent_set_owner_r(skb, asoc);
+
 	/* Embed the event fields inside the cloned skb.  */
-	event = sctp_skb2event(skb);
+	event = (sctp_ulpevent_t *) skb->cb;
 
 	/* Initialize event with flags 0.  */
-	sctp_ulpevent_init(event, 0);
-
-	sctp_ulpevent_receive_data(event, asoc);
-
-	event->stream = ntohs(chunk->subh.data_hdr->stream);
-	event->ssn = ntohs(chunk->subh.data_hdr->ssn);
-	event->ppid = chunk->subh.data_hdr->ppid;
-	if (chunk->chunk_hdr->flags & SCTP_DATA_UNORDERED) {
-		event->flags |= MSG_UNORDERED;
-		event->cumtsn = sctp_tsnmap_get_ctsn(&asoc->peer.tsn_map);
-	}
-	event->tsn = ntohl(chunk->subh.data_hdr->tsn);
-	event->msg_flags |= chunk->chunk_hdr->flags;
-	event->iif = sctp_chunk_iif(chunk);
-
-fail:
-	return event;
-}
-
-/* Create a partial delivery related event.
- *
- * 5.3.1.7 SCTP_PARTIAL_DELIVERY_EVENT
- *
- *   When a reciever is engaged in a partial delivery of a
- *   message this notification will be used to inidicate
- *   various events.
- */
-struct sctp_ulpevent *sctp_ulpevent_make_pdapi(
-	const struct sctp_association *asoc, __u32 indication, int gfp)
-{
-	struct sctp_ulpevent *event;
-	struct sctp_pdapi_event *pd;
-	struct sk_buff *skb;
-
-	event = sctp_ulpevent_new(sizeof(struct sctp_pdapi_event),
-				  MSG_NOTIFICATION, gfp);
+	event = sctp_ulpevent_init(event, skb, 0); 
 	if (!event)
-		goto fail;
+		goto fail_init;
 
-	skb = sctp_event2skb(event);
-	pd = (struct sctp_pdapi_event *)
-		skb_put(skb, sizeof(struct sctp_pdapi_event));
+	event->malloced = 1;
 
-	/* pdapi_type
-	 *   It should be SCTP_PARTIAL_DELIVERY_EVENT
-	 *
-	 * pdapi_flags: 16 bits (unsigned integer)
-	 *   Currently unused.
-	 */
-	pd->pdapi_type = SCTP_PARTIAL_DELIVERY_EVENT;
-	pd->pdapi_flags = 0;
-
-	/* pdapi_length: 32 bits (unsigned integer)
-	 *
-	 * This field is the total length of the notification data, including
-	 * the notification header.  It will generally be sizeof (struct
-	 * sctp_pdapi_event).
-	 */
-	pd->pdapi_length = sizeof(struct sctp_pdapi_event);
-
-        /*  pdapi_indication: 32 bits (unsigned integer)
-	 *
-	 * This field holds the indication being sent to the application.
-	 */
-	pd->pdapi_indication = indication;
-
-	/*  pdapi_assoc_id: sizeof (sctp_assoc_t)
-	 *
-	 * The association id field, holds the identifier for the association.
-	 */
-	sctp_ulpevent_set_owner(event, asoc);
-	pd->pdapi_assoc_id = sctp_assoc2id(asoc);
-
-	return event;
-fail:
-	return NULL;
-}
-
-/* Return the notification type, assuming this is a notification
- * event.
- */
-__u16 sctp_ulpevent_get_notification_type(const struct sctp_ulpevent *event)
-{
-	union sctp_notification *notification;
-	struct sk_buff *skb;
-
-	skb = sctp_event2skb((struct sctp_ulpevent *)event);
-	notification = (union sctp_notification *) skb->data;
-	return notification->sn_header.sn_type;
-}
-
-/* Copy out the sndrcvinfo into a msghdr.  */
-void sctp_ulpevent_read_sndrcvinfo(const struct sctp_ulpevent *event,
-				   struct msghdr *msghdr)
-{
-	struct sctp_sndrcvinfo sinfo;
-
-	if (sctp_ulpevent_is_notification(event))
-		return;
+	info = (struct sctp_sndrcvinfo *) &event->sndrcvinfo;
 
 	/* Sockets API Extensions for SCTP
- 	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
- 	 *
- 	 * sinfo_stream: 16 bits (unsigned integer)
- 	 *
- 	 * For recvmsg() the SCTP stack places the message's stream number in
- 	 * this value.
- 	*/
-	sinfo.sinfo_stream = event->stream;
-	/* sinfo_ssn: 16 bits (unsigned integer)
+	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
+	 *
+	 * sinfo_stream: 16 bits (unsigned integer)
+	 *
+	 * For recvmsg() the SCTP stack places the message's stream number in
+	 * this value.
+	 */
+	info->sinfo_stream = ntohs(chunk->subh.data_hdr->stream);
+
+	/* Sockets API Extensions for SCTP
+	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
+	 *
+	 * sinfo_ssn: 16 bits (unsigned integer)
 	 *
 	 * For recvmsg() this value contains the stream sequence number that
 	 * the remote endpoint placed in the DATA chunk.  For fragmented
 	 * messages this is the same number for all deliveries of the message
 	 * (if more than one recvmsg() is needed to read the message).
 	 */
-	sinfo.sinfo_ssn = event->ssn;
-	/* sinfo_ppid: 32 bits (unsigned integer)
+	info->sinfo_ssn = ntohs(chunk->subh.data_hdr->ssn);
+
+        /* Sockets API Extensions for SCTP
+	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
+	 *
+	 * sinfo_ppid: 32 bits (unsigned integer)
 	 *
 	 * In recvmsg() this value is
 	 * the same information that was passed by the upper layer in the peer
@@ -815,8 +686,12 @@ void sctp_ulpevent_read_sndrcvinfo(const struct sctp_ulpevent *event,
 	 * for and this information is passed opaquely by the SCTP stack from
 	 * one end to the other.
 	 */
-	sinfo.sinfo_ppid = event->ppid;
-	/* sinfo_flags: 16 bits (unsigned integer)
+	info->sinfo_ppid = ntohl(chunk->subh.data_hdr->ppid);
+
+	/* Sockets API Extensions for SCTP
+	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
+	 *
+	 * sinfo_flags: 16 bits (unsigned integer)
 	 *
 	 * This field may contain any of the following flags and is composed of
 	 * a bitwise OR of these values.
@@ -826,71 +701,71 @@ void sctp_ulpevent_read_sndrcvinfo(const struct sctp_ulpevent *event,
 	 * MSG_UNORDERED - This flag is present when the message was sent
 	 *                 non-ordered.
 	 */
-	sinfo.sinfo_flags = event->flags;
-	/* sinfo_tsn: 32 bit (unsigned integer)
-	 *
-	 * For the receiving side, this field holds a TSN that was 
-	 * assigned to one of the SCTP Data Chunks.
+	if (chunk->chunk_hdr->flags & SCTP_DATA_UNORDERED)
+		info->sinfo_flags |= MSG_UNORDERED;
+
+	/* FIXME:  For reassembly, we need to have the fragmentation bits.
+	 * This really does not belong in the event structure, but
+	 * its difficult to fix everything at the same time.   Eventually,
+	 * we should create and skb based chunk structure.   This structure
+	 * storage can be converted to an event.  --jgrimm
 	 */
-	sinfo.sinfo_tsn = event->tsn;
-	/* sinfo_cumtsn: 32 bit (unsigned integer)
+	event->chunk_flags = chunk->chunk_hdr->flags;
+
+	/* With -04 draft, tsn moves into sndrcvinfo. */
+	info->sinfo_tsn = ntohl(chunk->subh.data_hdr->tsn);
+
+	/* Context is not used on receive. */
+	info->sinfo_context = 0;
+
+	/* Sockets API Extensions for SCTP
+	 * Section 5.2.2 SCTP Header Information Structure (SCTP_SNDRCV)
 	 *
-	 * This field will hold the current cumulative TSN as
-	 * known by the underlying SCTP layer.  Note this field is
-	 * ignored when sending and only valid for a receive
-	 * operation when sinfo_flags are set to MSG_UNORDERED.
-	 */
-	sinfo.sinfo_cumtsn = event->cumtsn;
-	/* sinfo_assoc_id: sizeof (sctp_assoc_t)
+	 * sinfo_assoc_id: sizeof (sctp_assoc_t)
 	 *
 	 * The association handle field, sinfo_assoc_id, holds the identifier
 	 * for the association announced in the COMMUNICATION_UP notification.
 	 * All notifications for a given association have the same identifier.
-	 * Ignored for one-to-one style sockets.
+	 * Ignored for TCP-style sockets.
 	 */
-	sinfo.sinfo_assoc_id = sctp_assoc2id(event->asoc);
+	info->sinfo_assoc_id = sctp_assoc2id(asoc);
 
-	/* These fields are not used while receiving. */
-	sinfo.sinfo_context = 0;
-	sinfo.sinfo_timetolive = 0;
+	return event;
 
-	put_cmsg(msghdr, IPPROTO_SCTP, SCTP_SNDRCV,
-		 sizeof(struct sctp_sndrcvinfo), (void *)&sinfo);
+fail_init:
+	kfree_skb(skb);
+
+fail:
+	return NULL;
 }
 
-/* Do accounting for bytes received and hold a reference to the association
- * for each skb.
+/* Return the notification type, assuming this is a notification
+ * event.
  */
-static void sctp_ulpevent_receive_data(struct sctp_ulpevent *event,
-				       struct sctp_association *asoc)
+__u16 sctp_ulpevent_get_notification_type(const sctp_ulpevent_t *event)
 {
-	struct sk_buff *skb, *frag;
+	union sctp_notification *notification;
 
-	skb = sctp_event2skb(event);
-	/* Set the owner and charge rwnd for bytes received.  */
-	sctp_ulpevent_set_owner(event, asoc);
-	sctp_assoc_rwnd_decrease(asoc, skb_headlen(skb));
+	notification = (union sctp_notification *) event->parent->data;
+	return notification->h.sn_type;
+}
 
-	if (!skb->data_len)
-		return;
-
-	/* Note:  Not clearing the entire event struct as this is just a
-	 * fragment of the real event.  However, we still need to do rwnd
-	 * accounting.
-	 * In general, the skb passed from IP can have only 1 level of
-	 * fragments. But we allow multiple levels of fragments. 
-	 */
-	for (frag = skb_shinfo(skb)->frag_list; frag; frag = frag->next) {
-		sctp_ulpevent_receive_data(sctp_skb2event(frag), asoc);
+/* Copy out the sndrcvinfo into a msghdr.  */
+void sctp_ulpevent_read_sndrcvinfo(const sctp_ulpevent_t *event,
+				   struct msghdr *msghdr)
+{
+	if (!sctp_ulpevent_is_notification(event)) {
+		put_cmsg(msghdr, IPPROTO_SCTP, SCTP_SNDRCV,
+			 sizeof(struct sctp_sndrcvinfo),
+			 (void *) &event->sndrcvinfo);
 	}
 }
 
-/* Do accounting for bytes just read by user and release the references to
- * the association.
- */ 
-static void sctp_ulpevent_release_data(struct sctp_ulpevent *event)
+/* Do accounting for bytes just read by user.  */
+static void sctp_rcvmsg_rfree(struct sk_buff *skb)
 {
-	struct sk_buff *skb, *frag;
+	sctp_association_t *asoc;
+	sctp_ulpevent_t *event;
 
 	/* Current stack structures assume that the rcv buffer is
 	 * per socket.   For UDP style sockets this is not true as
@@ -898,45 +773,68 @@ static void sctp_ulpevent_release_data(struct sctp_ulpevent *event)
 	 * Use the local private area of the skb to track the owning
 	 * association.
 	 */
-
-	skb = sctp_event2skb(event);
-	sctp_assoc_rwnd_increase(event->asoc, skb_headlen(skb));
-
-	if (!skb->data_len)
-		goto done;
-
-	/* Don't forget the fragments. */
-	for (frag = skb_shinfo(skb)->frag_list; frag; frag = frag->next) {
-		/* NOTE:  skb_shinfos are recursive. Although IP returns
-		 * skb's with only 1 level of fragments, SCTP reassembly can
-		 * increase the levels.
-		 */
-		sctp_ulpevent_release_data(sctp_skb2event(frag));
+	event = (sctp_ulpevent_t *) skb->cb;
+	asoc = event->asoc;
+	if (asoc->rwnd_over) {
+		if (asoc->rwnd_over >= skb->len) {
+			asoc->rwnd_over -= skb->len;
+		} else {
+			asoc->rwnd += (skb->len - asoc->rwnd_over);
+			asoc->rwnd_over = 0;
+		}
+	} else {
+		asoc->rwnd += skb->len;
 	}
 
-done:
-	sctp_ulpevent_release_owner(event);
+	SCTP_DEBUG_PRINTK("rwnd increased by %d to (%u, %u)\n",
+			  skb->len, asoc->rwnd, asoc->rwnd_over);
+
+	sctp_association_put(asoc);
 }
 
-/* Free a ulpevent that has an owner.  It includes releasing the reference
- * to the owner, updating the rwnd in case of a DATA event and freeing the
- * skb.
- * See comments in sctp_stub_rfree().
- */
-void sctp_ulpevent_free(struct sctp_ulpevent *event)
+/* Charge receive window for bytes recieved.  */
+static void sctp_ulpevent_set_owner_r(struct sk_buff *skb, sctp_association_t *asoc)
 {
-	if (sctp_ulpevent_is_notification(event))
-		sctp_ulpevent_release_owner(event);
-	else
-		sctp_ulpevent_release_data(event);
+	sctp_ulpevent_t *event;
 
-	kfree_skb(sctp_event2skb(event));
+	/* The current stack structures assume that the rcv buffer is
+	 * per socket.  For UDP-style sockets this is not true as
+	 * multiple associations may be on a single UDP-style socket.
+	 * We use the local private area of the skb to track the owning
+	 * association.
+	 */
+	sctp_association_hold(asoc);
+	skb->sk = asoc->base.sk;
+	event = (sctp_ulpevent_t *) skb->cb;
+	event->asoc = asoc;
+
+	skb->destructor = sctp_rcvmsg_rfree;
+
+	SCTP_ASSERT(asoc->rwnd, "rwnd zero", return);
+	SCTP_ASSERT(!asoc->rwnd_over, "rwnd_over not zero", return);
+	if (asoc->rwnd >= skb->len) {
+		asoc->rwnd -= skb->len;
+	} else {
+		asoc->rwnd_over = skb->len - asoc->rwnd;
+		asoc->rwnd = 0;
+	}
+
+	SCTP_DEBUG_PRINTK("rwnd decreased by %d to (%u, %u)\n",
+			  skb->len, asoc->rwnd, asoc->rwnd_over);
 }
 
-/* Purge the skb lists holding ulpevents. */
-void sctp_queue_purge_ulpevents(struct sk_buff_head *list)
-{
-	struct sk_buff *skb;
-	while ((skb = skb_dequeue(list)) != NULL)
-		sctp_ulpevent_free(sctp_skb2event(skb));
-}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
